@@ -71,6 +71,15 @@ enum {
 	MSC_CTR_BSSMAP_RX_DT1_DTAP_ERROR,
 	MSC_CTR_BSSMAP_RX_DT1_PERFORM_LOCATION_REQUEST,
 	MSC_CTR_BSSMAP_RX_DT1_PERFORM_LOCATION_ABORT,
+	MSC_CTR_BSSMAP_RX_DT1_VGCS_VBS_SETUP,
+	MSC_CTR_BSSMAP_RX_DT1_VGCS_VBS_ASSIGN_RQST,
+	MSC_CTR_BSSMAP_RX_DT1_UPLINK_RQST_ACKNOWLEDGE,
+	MSC_CTR_BSSMAP_RX_DT1_UPLINK_REJECT_CMD,
+	MSC_CTR_BSSMAP_RX_DT1_UPLINK_RELEASE_CMD,
+	MSC_CTR_BSSMAP_RX_DT1_UPLINK_SEIZED_CMD,
+	MSC_CTR_BSSMAP_RX_DT1_VGCS_ADDL_INFO,
+	MSC_CTR_BSSMAP_RX_DT1_VGCS_SMS,
+	MSC_CTR_BSSMAP_RX_DT1_NOTIFICATION_DATA,
 
 	/* Tx message counters (per connection type) */
 	MSC_CTR_BSSMAP_TX_BSS_MANAGEMENT,
@@ -86,8 +95,8 @@ enum {
 	MSC_CTR_BSSMAP_TX_UDT_RESET_ACK,
 	MSC_CTR_BSSMAP_TX_DT1_CLEAR_RQST,
 	MSC_CTR_BSSMAP_TX_DT1_CLEAR_COMPLETE,
-	MSC_CTR_BSSMAP_TX_DT1_ASSIGMENT_FAILURE,
-	MSC_CTR_BSSMAP_TX_DT1_ASSIGMENT_COMPLETE,
+	MSC_CTR_BSSMAP_TX_DT1_ASSIGNMENT_FAILURE,
+	MSC_CTR_BSSMAP_TX_DT1_ASSIGNMENT_COMPLETE,
 	MSC_CTR_BSSMAP_TX_DT1_SAPI_N_REJECT,
 	MSC_CTR_BSSMAP_TX_DT1_CIPHER_COMPLETE,
 	MSC_CTR_BSSMAP_TX_DT1_CIPHER_REJECT,
@@ -102,6 +111,17 @@ enum {
 	MSC_CTR_BSSMAP_TX_DT1_DTAP,
 	MSC_CTR_BSSMAP_TX_DT1_PERFORM_LOCATION_RESPONSE_SUCCESS,
 	MSC_CTR_BSSMAP_TX_DT1_PERFORM_LOCATION_RESPONSE_FAILURE,
+	MSC_CTR_BSSMAP_TX_DT1_VGCS_VBS_SETUP_ACK,
+	MSC_CTR_BSSMAP_TX_DT1_VGCS_VBS_SETUP_REFUSE,
+	MSC_CTR_BSSMAP_TX_DT1_VGCS_VBS_ASSIGN_RESULT,
+	MSC_CTR_BSSMAP_TX_DT1_VGCS_VBS_ASSIGN_FAIL,
+	MSC_CTR_BSSMAP_TX_DT1_VGCS_VBS_QUEUING_INDICATION,
+	MSC_CTR_BSSMAP_TX_DT1_UPLINK_RQST,
+	MSC_CTR_BSSMAP_TX_DT1_VGCS_VBS_ASSIGNMENT_STATUS,
+	MSC_CTR_BSSMAP_TX_DT1_VGCS_VBS_AREA_CELL_INFO,
+	MSC_CTR_BSSMAP_TX_DT1_UPLINK_RQST_CONFIRMATION,
+	MSC_CTR_BSSMAP_TX_DT1_UPLINK_RELEASE_INDICATION,
+	MSC_CTR_BSSMAP_TX_DT1_UPLINK_APP_DATA,
 
 	MSC_CTR_MSCPOOL_SUBSCR_NEW,
 	MSC_CTR_MSCPOOL_SUBSCR_REATTACH,
@@ -133,8 +153,13 @@ struct bsc_msc_data {
 	/* audio codecs */
 	struct gsm48_multi_rate_conf amr_conf;
 	bool amr_octet_aligned;
-	struct gsm_audio_support **audio_support;
+
+	/* Practically, there can be only 5 entries in osmo-bsc: FR1 FR2 FR3 HR1 HR3. Historically, osmo-bsc allowed
+	 * *any* number of entries -- we should not break too strictly on old cfg files. Theoretically, there should be
+	 * room for FR1 to FR7 plus HR1 to HR7 less HR2. Let's just give ample room for all these aspects: */
+	struct gsm_audio_support audio_support[16];
 	int audio_length;
+
 	enum bsc_lcls_mode lcls_mode;
 	bool lcls_codec_mismatch_allow;
 
