@@ -300,7 +300,7 @@ struct gsm_bts_model {
 	/* (Optional) function for sending default MS/BS Power Control paramaters */
 	int (*power_ctrl_send_def_params)(const struct gsm_bts_trx *trx);
 	/* (Optional) function for toggling BCCH carrier power reduction operation */
-	int (*power_ctrl_set_c0_power_red)(const struct gsm_bts *bts, const uint8_t red);
+	int (*power_ctrl_send_c0_power_red)(const struct gsm_bts *bts, const uint8_t red);
 
 	void (*config_write_bts)(struct vty *vty, struct gsm_bts *bts);
 	void (*config_write_trx)(struct vty *vty, struct gsm_bts_trx *trx);
@@ -523,6 +523,7 @@ struct gsm_bts {
 	bool chan_alloc_chan_req_reverse;
 	bool chan_alloc_assignment_reverse;
 	bool chan_alloc_handover_reverse;
+	bool chan_alloc_vgcs_reverse;
 
 	/* Whether to use dynamic allocation mode for assignment */
 	bool chan_alloc_assignment_dynamic;
@@ -568,6 +569,11 @@ struct gsm_bts {
 			uint16_t scramble_list[MAX_EARFCN_LIST];
 		} data;
 	} si_common;
+	/* NCH (Notification Channel) related parameters */
+	struct {
+		uint8_t num_blocks;	/* NCH number of CCCH blocks (0 = no NCH) */
+		uint8_t first_block;	/* NCH first block number */
+	} nch;
 	bool early_classmark_allowed;
 	bool early_classmark_allowed_3g;
 	/* for testing only: Have an infinitely long radio link timeout */
@@ -600,10 +606,6 @@ struct gsm_bts {
 	/* osmux config: */
 	enum osmux_usage use_osmux;
 
-	/* PCU socket state */
-	char *pcu_sock_path;
-	struct pcu_sock_state *pcu_state;
-
 	struct rate_ctr_group *bts_ctrs;
 	struct osmo_stat_item_group *bts_statg;
 
@@ -678,77 +680,89 @@ struct gsm_bts {
 #define GSM_BTS_SI(bts, i)     (void *)((bts)->si_buf[i][0])
 
 /* this actually refers to the IPA transport, not the BTS model */
-static inline int is_ipaccess_bts(const struct gsm_bts *bts)
+static inline bool is_ipa_abisip_bts(const struct gsm_bts *bts)
 {
 	switch (bts->type) {
 	case GSM_BTS_TYPE_NANOBTS:
 	case GSM_BTS_TYPE_OSMOBTS:
-		return 1;
+		return true;
 	default:
 		break;
 	}
-	return 0;
+	return false;
 }
 
-static inline int is_osmobts(const struct gsm_bts *bts)
+static inline bool is_osmobts(const struct gsm_bts *bts)
 {
 	switch (bts->type) {
 	case GSM_BTS_TYPE_OSMOBTS:
-		return 1;
+		return true;
 	default:
 		break;
 	}
-	return 0;
+	return false;
 }
 
-static inline int is_siemens_bts(const struct gsm_bts *bts)
+static inline bool is_siemens_bts(const struct gsm_bts *bts)
 {
 	switch (bts->type) {
 	case GSM_BTS_TYPE_BS11:
-		return 1;
+		return true;
 	default:
 		break;
 	}
 
-	return 0;
+	return false;
 }
 
-static inline int is_nokia_bts(const struct gsm_bts *bts)
+static inline bool is_nokia_bts(const struct gsm_bts *bts)
 {
 	switch (bts->type) {
 	case GSM_BTS_TYPE_NOKIA_SITE:
-		return 1;
+		return true;
 	default:
 		break;
 	}
 
-	return 0;
+	return false;
 }
 
-static inline int is_ericsson_bts(const struct gsm_bts *bts)
+static inline bool is_ericsson_bts(const struct gsm_bts *bts)
 {
 	switch (bts->type) {
 	case GSM_BTS_TYPE_RBS2000:
-		return 1;
+		return true;
 	default:
 		break;
 	}
 
-	return 0;
+	return false;
 }
 
-static inline int is_e1_bts(const struct gsm_bts *bts)
+static inline bool is_e1_bts(const struct gsm_bts *bts)
 {
 	switch (bts->type) {
 	case GSM_BTS_TYPE_BS11:
 	case GSM_BTS_TYPE_RBS2000:
 	case GSM_BTS_TYPE_NOKIA_SITE:
-		return 1;
+		return true;
 	default:
 		break;
 	}
 
-	return 0;
+	return false;
+}
+
+static inline bool bsc_co_located_pcu(const struct gsm_bts *bts)
+{
+	switch (bts->type) {
+	case GSM_BTS_TYPE_RBS2000:
+		return true;
+	default:
+		break;
+	}
+
+	return false;
 }
 
 static inline const struct osmo_location_area_id *bts_lai(struct gsm_bts *bts)
@@ -814,6 +828,7 @@ void gsm_bts_all_ts_dispatch(struct gsm_bts *bts, uint32_t ts_ev, void *data);
 
 int gsm_bts_set_system_infos(struct gsm_bts *bts);
 
+int gsm_bts_send_c0_power_red(const struct gsm_bts *bts, const uint8_t red);
 int gsm_bts_set_c0_power_red(struct gsm_bts *bts, const uint8_t red);
 
 void gsm_bts_stats_reset(struct gsm_bts *bts);
