@@ -526,9 +526,9 @@ struct gsm_abis_mo {
 	struct abis_om_obj_inst obj_inst;
 	const char *name;
 	struct gsm_nm_state nm_state;
+	struct tlv_parsed *nm_attr;
 	struct gsm_bts *bts;
 	struct osmo_fsm_inst *fi;
-	bool sw_act_rep_received;
 	bool opstart_sent;
 	bool adm_unlock_sent;
 	bool get_attr_sent;
@@ -538,12 +538,6 @@ struct gsm_abis_mo {
 	bool rsl_connect_sent;
 	bool rsl_connect_ack_received;
 	bool force_rf_lock;
-	/* vendor specific fields below */
-	union {
-		struct {
-			uint8_t obj_version;
-		} ipaccess;
-	};
 };
 
 /* Ericsson OM2000 Managed Object */
@@ -674,6 +668,7 @@ struct gsm_bts_trx_ts {
 	bool is_rsl_ready;
 
 	struct gsm_abis_mo mo;
+	struct tlv_parsed nm_attr;
 	uint8_t nm_chan_comb;
 	int tsc;		/* -1 == use BTS TSC */
 
@@ -897,8 +892,6 @@ void gsm_abis_mo_reset(struct gsm_abis_mo *mo);
 void gsm_mo_init(struct gsm_abis_mo *mo, struct gsm_bts *bts,
 		 uint8_t obj_class, uint8_t p1, uint8_t p2, uint8_t p3);
 
-struct gsm_abis_mo *gsm_objclass2mo(struct gsm_bts *bts, uint8_t obj_class,
-				    const struct abis_om_obj_inst *obj_inst);
 struct gsm_nm_state *
 gsm_objclass2nmstate(struct gsm_bts *bts, uint8_t obj_class,
 		 const struct abis_om_obj_inst *obj_inst);
@@ -1068,7 +1061,6 @@ struct gsm_network {
 
 	/* PCU socket state */
 	char *pcu_sock_path;
-	unsigned int pcu_sock_wqueue_len_max;
 	struct pcu_sock_state *pcu_state;
 };
 
