@@ -42,7 +42,7 @@
 
 static struct osmo_fsm assignment_fsm;
 
-struct gsm_subscriber_connection *assignment_fi_conn(struct osmo_fsm_inst *fi)
+static struct gsm_subscriber_connection *assignment_fi_conn(struct osmo_fsm_inst *fi)
 {
 	OSMO_ASSERT(fi);
 	OSMO_ASSERT(fi->fsm == &assignment_fsm);
@@ -54,7 +54,7 @@ static const struct osmo_tdef_state_timeout assignment_fsm_timeouts[32] = {
 	[ASSIGNMENT_ST_WAIT_LCHAN_ACTIVE] = { .T = 10 },
 	[ASSIGNMENT_ST_WAIT_RR_ASS_COMPLETE] = { .keep_timer = true },
 	[ASSIGNMENT_ST_WAIT_LCHAN_ESTABLISHED] = { .keep_timer = true },
-	[ASSIGNMENT_ST_WAIT_MGW_ENDPOINT_TO_MSC] = { .T = 23042 },
+	[ASSIGNMENT_ST_WAIT_MGW_ENDPOINT_TO_MSC] = { .T = -9 },
 };
 
 /* Transition to a state, using the T timer defined in assignment_fsm_timeouts.
@@ -152,7 +152,8 @@ static void on_assignment_failure(struct gsm_subscriber_connection *conn)
 		if (!resp) {
 			LOG_ASSIGNMENT(conn, LOGL_ERROR, "Unable to compose BSSMAP Assignment Failure message\n");
 		} else {
-			rate_ctr_inc(rate_ctr_group_get_ctr(conn->sccp.msc->msc_ctrs, MSC_CTR_BSSMAP_TX_DT1_ASSIGMENT_FAILURE));
+			rate_ctr_inc(rate_ctr_group_get_ctr(conn->sccp.msc->msc_ctrs,
+				     MSC_CTR_BSSMAP_TX_DT1_ASSIGNMENT_FAILURE));
 			gscon_sigtran_send(conn, resp);
 		}
 	}
@@ -164,7 +165,7 @@ static void on_assignment_failure(struct gsm_subscriber_connection *conn)
 	}
 }
 
-static void _gsm0808_ass_compl_extend_osmux(struct msgb *msg, uint8_t cid)
+void bssap_extend_osmux(struct msgb *msg, uint8_t cid)
 {
 	OSMO_ASSERT(msg->l3h[1] == msgb_l3len(msg) - 2); /*TL not in len */
 	msgb_tv_put(msg, GSM0808_IE_OSMO_OSMUX_CID, cid);
@@ -185,6 +186,11 @@ static void send_assignment_complete(struct gsm_subscriber_connection *conn)
 	struct gsm_lchan *lchan = conn->lchan;
 	struct osmo_fsm_inst *fi = conn->fi;
 
+	if (!lchan) {
+		assignment_fail(GSM0808_CAUSE_EQUIPMENT_FAILURE, "Assignment interrupted: primary lchan lost");
+		return;
+	}
+
 	chosen_channel = gsm0808_chosen_channel(lchan->type, lchan->current_ch_mode_rate.chan_mode);
 	if (!chosen_channel) {
 		assignment_fail(GSM0808_CAUSE_EQUIPMENT_FAILURE,
@@ -194,21 +200,26 @@ static void send_assignment_complete(struct gsm_subscriber_connection *conn)
 		return;
 	}
 
-	/* Generate voice related fields */
-	if (conn->assignment.requires_voice_stream) {
+	if (gscon_is_aoip(conn) && bsc_chan_ind_requires_rtp_stream(conn->assignment.ch_indctr)) {
+		if (!osmo_mgcpc_ep_ci_get_crcx_info_to_sockaddr(conn->user_plane.mgw_endpoint_ci_msc,
+								&addr_local)) {
+			assignment_fail(GSM0808_CAUSE_EQUIPMENT_FAILURE,
+					"Unable to compose RTP address of MGW -> MSC");
+			return;
+		}
+		addr_local_p = &addr_local;
+	}
+
+	/* Generate rtp related fields */
+	switch (conn->assignment.ch_indctr) {
+	case GSM0808_CHAN_SPEECH:
 		perm_spch = gsm0808_permitted_speech(lchan->type, lchan->current_ch_mode_rate.chan_mode);
 
-		if (gscon_is_aoip(conn)) {
-			if (!osmo_mgcpc_ep_ci_get_crcx_info_to_sockaddr(conn->user_plane.mgw_endpoint_ci_msc,
-									&addr_local)) {
-				assignment_fail(GSM0808_CAUSE_EQUIPMENT_FAILURE,
-						"Unable to compose RTP address of MGW -> MSC");
-				return;
-			}
-			addr_local_p = &addr_local;
-		}
+		/* below is AoIP specific logic */
+		if (!gscon_is_aoip(conn))
+			break;
 
-		if (gscon_is_aoip(conn) && conn->assignment.req.use_osmux) {
+		if (conn->assignment.req.use_osmux) {
 			if (!osmo_mgcpc_ep_ci_get_crcx_info_to_osmux_cid(conn->user_plane.mgw_endpoint_ci_msc,
 									 &osmux_cid)) {
 				assignment_fail(GSM0808_CAUSE_EQUIPMENT_FAILURE,
@@ -217,14 +228,28 @@ static void send_assignment_complete(struct gsm_subscriber_connection *conn)
 			}
 		}
 
-		/* Only AoIP networks include a speech codec (choosen) in the
-		 * assignment complete message. */
-		if (gscon_is_aoip(conn)) {
-			/* Extrapolate speech codec from speech mode */
-			gsm0808_speech_codec_from_chan_type(&sc, perm_spch);
-			sc.cfg = conn->lchan->current_ch_mode_rate.s15_s0;
-			sc_ptr = &sc;
-		}
+		/* Extrapolate speech codec from speech mode */
+		gsm0808_speech_codec_from_chan_type(&sc, perm_spch);
+		sc.cfg = conn->lchan->current_ch_mode_rate.s15_s0;
+		sc_ptr = &sc;
+		break;
+	case GSM0808_CHAN_DATA:
+		/* below is AoIP specific logic */
+		if (!gscon_is_aoip(conn))
+			break;
+
+		/* The coding of Speech Codec Element for the CSData Codec Type
+		 * is defined in 3GPP TS 48.008 section 3.2.2.103 */
+		sc = (struct gsm0808_speech_codec) {
+			.pi = true, /* PI indicates CSDoIP support */
+			.pt = false, /* PT indicates CSDoTDM support */
+			.type = GSM0808_SCT_CSD,
+			.cfg = 0, /* R2/R3 not set (redundancy not supported) */
+		};
+		sc_ptr = &sc;
+		break;
+	default:
+		break;
 	}
 
 	resp = gsm0808_create_ass_compl2(lchan->abis_ip.ass_compl.rr_cause,
@@ -238,11 +263,11 @@ static void send_assignment_complete(struct gsm_subscriber_connection *conn)
 		return;
 	}
 
-	if (gscon_is_aoip(conn) && conn->assignment.requires_voice_stream &&
+	if (gscon_is_aoip(conn) && bsc_chan_ind_requires_rtp_stream(conn->assignment.ch_indctr) &&
 	    conn->assignment.req.use_osmux)
-		_gsm0808_ass_compl_extend_osmux(resp, osmux_cid);
+		bssap_extend_osmux(resp, osmux_cid);
 
-	rate_ctr_inc(rate_ctr_group_get_ctr(conn->sccp.msc->msc_ctrs, MSC_CTR_BSSMAP_TX_DT1_ASSIGMENT_COMPLETE));
+	rate_ctr_inc(rate_ctr_group_get_ctr(conn->sccp.msc->msc_ctrs, MSC_CTR_BSSMAP_TX_DT1_ASSIGNMENT_COMPLETE));
 	rc = gscon_sigtran_send(conn, resp);
 	if (rc) {
 		assignment_fail(GSM0808_CAUSE_EQUIPMENT_FAILURE,
@@ -255,9 +280,11 @@ static void send_assignment_complete(struct gsm_subscriber_connection *conn)
 static void assignment_success(struct gsm_subscriber_connection *conn)
 {
 	struct gsm_bts *bts;
-	bool lchan_changed = (conn->assignment.new_lchan != NULL);
+	bool lchan_changed = (conn->assignment.new_lchan != NULL && !conn->assignment.req.vgcs);
 
-	/* Take on the new lchan. If there only was a Channel Mode Modify, then there is no new lchan to take on. */
+	/* Take on the new lchan. If there only was a Channel Mode Modify, then there is no new lchan to take on.
+	 * In case of VGCS/VBS channel, the assignment is handled by its state machine. This subscriber connection will
+	 * be released by MSC. */
 	if (lchan_changed) {
 		gscon_change_primary_lchan(conn, conn->assignment.new_lchan);
 
@@ -303,7 +330,7 @@ static void assignment_success(struct gsm_subscriber_connection *conn)
 	osmo_fsm_inst_term(conn->assignment.fi, OSMO_FSM_TERM_REGULAR, 0);
 }
 
-static void assignment_fsm_update_id(struct gsm_subscriber_connection *conn)
+void assignment_fsm_update_id(struct gsm_subscriber_connection *conn)
 {
 	/* Assignment can do a new channel activation, in which case new_lchan points at the new lchan.
 	 * Or assignment can Channel Mode Modify the already used lchan, in which case new_lchan == NULL. */
@@ -313,15 +340,16 @@ static void assignment_fsm_update_id(struct gsm_subscriber_connection *conn)
 		return;
 	}
 
-	osmo_fsm_inst_update_id_f(conn->assignment.fi, "%s_%u-%u-%u-%s%s%s-%s%u",
-				  conn->fi->id,
-				  new_lchan->ts->trx->bts->nr, new_lchan->ts->trx->nr, new_lchan->ts->nr,
-				  gsm_pchan_id(new_lchan->ts->pchan_on_init),
-				  (new_lchan->ts->pchan_on_init == new_lchan->ts->pchan_is)? "" : "as",
-				  (new_lchan->ts->pchan_on_init == new_lchan->ts->pchan_is)? ""
-					  : gsm_pchan_id(new_lchan->ts->pchan_is),
-				  new_lchan->vamos.is_secondary ? "shadow" : "",
-				  new_lchan->nr - (new_lchan->vamos.is_secondary ? new_lchan->ts->max_primary_lchans : 0));
+	osmo_fsm_inst_update_id_f_sanitize(conn->assignment.fi, '_', "%s_%u-%u-%u-%s%s%s-%s%u",
+					   conn->fi->id,
+					   new_lchan->ts->trx->bts->nr, new_lchan->ts->trx->nr, new_lchan->ts->nr,
+					   gsm_pchan_name(new_lchan->ts->pchan_on_init),
+					   (new_lchan->ts->pchan_on_init == new_lchan->ts->pchan_is) ? "" : "as",
+					   (new_lchan->ts->pchan_on_init == new_lchan->ts->pchan_is) ?
+						"" : gsm_pchan_name(new_lchan->ts->pchan_is),
+					   new_lchan->vamos.is_secondary ? "shadow" : "",
+					   new_lchan->nr - (new_lchan->vamos.is_secondary ?
+							    new_lchan->ts->max_primary_lchans : 0));
 }
 
 static bool lchan_type_compat_with_mode(enum gsm_chan_t type, const struct channel_mode_and_rate *ch_mode_rate)
@@ -364,48 +392,42 @@ static bool lchan_type_compat_with_mode(enum gsm_chan_t type, const struct chann
 	}
 }
 
-void assignment_fsm_init(void)
+static __attribute__((constructor)) void assignment_fsm_init(void)
 {
 	OSMO_ASSERT(osmo_fsm_register(&assignment_fsm) == 0);
 }
 
-static int check_requires_voice(bool *requires_voice, enum gsm48_chan_mode chan_mode)
+static int chan_mode_to_ch_indctr(enum gsm48_chan_mode chan_mode)
 {
-	*requires_voice = false;
-
 	switch (gsm48_chan_mode_to_non_vamos(chan_mode)) {
+	case GSM48_CMODE_DATA_14k5:
+	case GSM48_CMODE_DATA_12k0:
+	case GSM48_CMODE_DATA_6k0:
+	case GSM48_CMODE_DATA_3k6:
+		return GSM0808_CHAN_DATA;
 	case GSM48_CMODE_SPEECH_V1:
 	case GSM48_CMODE_SPEECH_EFR:
 	case GSM48_CMODE_SPEECH_AMR:
-		*requires_voice = true;
-		break;
+		return GSM0808_CHAN_SPEECH;
 	case GSM48_CMODE_SIGN:
-		*requires_voice = false;
-		break;
+		return GSM0808_CHAN_SIGN;
 	default:
 		return -EINVAL;
 	}
-
-	return 0;
 }
 
-/* Check if the incoming assignment requests requires a voice stream or not,
- * we will look at the preferred and the alternate channel mode and also make
- * sure that both are consistent. */
-static int check_requires_voice_stream(struct gsm_subscriber_connection *conn)
+/* Check if the incoming assignment request has a channel mode that is
+ * inconsistent with ch_indctr, e.g. GSM48_CMODE_DATA_14k5 and
+ * GSM0808_CHAN_SPEECH */
+static int check_chan_mode_rate_against_ch_indctr(struct gsm_subscriber_connection *conn)
 {
-	bool requires_voice_pref = false, requires_voice_alt;
 	struct assignment_request *req = &conn->assignment.req;
 	struct osmo_fsm_inst *fi = conn->fi;
-	int i, rc;
-
-	/* When the assignment request indicates that there is an alternate
-	 * rate available (e.g. "Full or Half rate channel, Half rate
-	 * preferred..."), then both must be either voice or either signalling,
-	 * a mismatch is not permitted */
+	int i;
+	int rc;
 
 	for (i = 0; i < req->n_ch_mode_rate; i++) {
-		rc = check_requires_voice(&requires_voice_alt, req->ch_mode_rate_list[i].chan_mode);
+		rc = chan_mode_to_ch_indctr(req->ch_mode_rate_list[i].chan_mode);
 		if (rc < 0) {
 			assignment_fail(GSM0808_CAUSE_REQ_CODEC_TYPE_OR_CONFIG_NOT_SUPP,
 					"Channel mode not supported (prev level %d): %s", i,
@@ -413,18 +435,15 @@ static int check_requires_voice_stream(struct gsm_subscriber_connection *conn)
 			return -EINVAL;
 		}
 
-		if (i==0)
-			requires_voice_pref = requires_voice_alt;
-		else if (requires_voice_alt != requires_voice_pref) {
+		if (rc != req->ch_indctr) {
 			assignment_fail(GSM0808_CAUSE_REQ_CODEC_TYPE_OR_CONFIG_NOT_SUPP,
-					"Requested a mix of Signalling and non-Signalling channel modes: %s != %s",
-					gsm48_chan_mode_name(req->ch_mode_rate_list[0].chan_mode),
-					gsm48_chan_mode_name(req->ch_mode_rate_list[i].chan_mode));
+					"Channel mode %s has ch_indctr %d, channel type has ch_indctr %d",
+					gsm48_chan_mode_name(req->ch_mode_rate_list[i].chan_mode),
+					rc, req->ch_indctr);
 			return -EINVAL;
 		}
 	}
 
-	conn->assignment.requires_voice_stream = requires_voice_pref;
 	return 0;
 }
 
@@ -470,6 +489,8 @@ static int _reassignment_request(enum assign_for assign_for, struct gsm_lchan *l
 			.present = (tsc >= 0),
 			.val = tsc,
 		},
+
+		.ch_indctr = chan_mode_to_ch_indctr(lchan->current_ch_mode_rate.chan_mode),
 	};
 
 	if (to_lchan)
@@ -528,10 +549,9 @@ void assignment_fsm_start(struct gsm_subscriber_connection *conn, struct gsm_bts
 
 	assignment_count(CTR_ASSIGNMENT_ATTEMPTED);
 
-	/* Check if we need a voice stream. If yes, set the appropriate struct
-	 * members in conn */
-	if (check_requires_voice_stream(conn) < 0)
+	if (check_chan_mode_rate_against_ch_indctr(conn) < 0)
 		return;
+	conn->assignment.ch_indctr = req->ch_indctr;
 
 	if (!req->target_lchan && reuse_existing_lchan(conn)) {
 		/* The already existing lchan is suitable for this mode */
@@ -570,7 +590,10 @@ void assignment_fsm_start(struct gsm_subscriber_connection *conn, struct gsm_bts
 		return;
 	}
 
-	if (req->target_lchan) {
+	if (req->vgcs) {
+		/* When assigning to a VGCS/VBS, the target lchan is already defined. */
+		conn->assignment.new_lchan = req->target_lchan;
+	} else if (req->target_lchan) {
 		bool matching_mode;
 
 		/* The caller already picked a target lchan to assign to. No need to try re-using the current lchan or
@@ -644,7 +667,9 @@ void assignment_fsm_start(struct gsm_subscriber_connection *conn, struct gsm_bts
 		       req->aoip ? "yes" : "no", req->msc_rtp_addr, req->msc_rtp_port,
 		       req->use_osmux ? "yes" : "no");
 
-	assignment_fsm_state_chg(ASSIGNMENT_ST_WAIT_LCHAN_ACTIVE);
+	/* Wait for lchan to become active before send assignment. In case of VGCS/VBS directly send assignment,
+	 * because the channel is already active. */
+	assignment_fsm_state_chg(req->vgcs ? ASSIGNMENT_ST_WAIT_RR_ASS_COMPLETE : ASSIGNMENT_ST_WAIT_LCHAN_ACTIVE);
 }
 
 static void assignment_fsm_wait_lchan_active_onenter(struct osmo_fsm_inst *fi, uint32_t prev_state)
@@ -656,15 +681,16 @@ static void assignment_fsm_wait_lchan_active_onenter(struct osmo_fsm_inst *fi, u
 		.for_conn = conn,
 		.ch_mode_rate = conn->assignment.selected_ch_mode_rate,
 		.encr = conn->lchan->encr,
-		.requires_voice_stream = conn->assignment.requires_voice_stream,
+		.ch_indctr = conn->assignment.ch_indctr,
 		.msc_assigned_cic = req->msc_assigned_cic,
 		.re_use_mgw_endpoint_from_lchan = conn->lchan,
 		.ta = conn->lchan->last_ta,
 		.ta_known = true,
 		.tsc_set = req->tsc_set,
 		.tsc = req->tsc,
-		.vamos = conn->assignment.new_lchan->vamos.is_secondary,
 	};
+	if (conn->assignment.new_lchan->vamos.is_secondary)
+		activ_info.type_for = LCHAN_TYPE_FOR_VAMOS;
 	lchan_activate(conn->assignment.new_lchan, &activ_info);
 }
 
@@ -772,7 +798,7 @@ static void assignment_fsm_wait_lchan_established(struct osmo_fsm_inst *fi, uint
 static void assignment_fsm_post_lchan_established(struct osmo_fsm_inst *fi)
 {
 	struct gsm_subscriber_connection *conn = assignment_fi_conn(fi);
-	if (conn->assignment.requires_voice_stream)
+	if (bsc_chan_ind_requires_rtp_stream(conn->assignment.ch_indctr))
 		assignment_fsm_state_chg(ASSIGNMENT_ST_WAIT_MGW_ENDPOINT_TO_MSC);
 	else
 		assignment_success(conn);
@@ -782,7 +808,7 @@ static void assignment_fsm_wait_mgw_endpoint_to_msc_onenter(struct osmo_fsm_inst
 {
 	struct gsm_subscriber_connection *conn = assignment_fi_conn(fi);
 
-	OSMO_ASSERT(conn->assignment.requires_voice_stream);
+	OSMO_ASSERT(bsc_chan_ind_requires_rtp_stream(conn->assignment.ch_indctr));
 
 	LOG_ASSIGNMENT(conn, LOGL_DEBUG,
 		       "Connecting MGW endpoint to the MSC's RTP port: %s:%u\n",
@@ -846,7 +872,7 @@ static void assignment_fsm_wait_lchan_modified_onenter(struct osmo_fsm_inst *fi,
 	struct lchan_modify_info modif_info = {
 		.modify_for = MODIFY_FOR_ASSIGNMENT,
 		.ch_mode_rate = conn->assignment.selected_ch_mode_rate,
-		.requires_voice_stream = conn->assignment.requires_voice_stream,
+		.ch_indctr = conn->assignment.ch_indctr,
 		.msc_assigned_cic = req->msc_assigned_cic,
 		/* keep previous training sequence code. TSC is always present, TSC Set may or may not be an explicit
 		 * value. */
@@ -987,7 +1013,7 @@ static int assignment_fsm_timer_cb(struct osmo_fsm_inst *fi)
 	return 0;
 }
 
-void assignment_fsm_cleanup(struct osmo_fsm_inst *fi, enum osmo_fsm_term_cause cause)
+static void assignment_fsm_cleanup(struct osmo_fsm_inst *fi, enum osmo_fsm_term_cause cause)
 {
 	struct gsm_subscriber_connection *conn = assignment_fi_conn(fi);
 	assignment_reset(conn);
