@@ -131,7 +131,7 @@ struct gsm_bts_model bts_model_nanobts = {
 			[NM_ATT_IPACC_REVOC_DATE] =	{ TLV_TYPE_TL16V },
 		},
 	},
-	.features_get_reported = true,
+	.features_get_reported = false,
 };
 
 
@@ -150,7 +150,7 @@ static int nm_statechg_event(int evt, struct nm_statechg_signal_data *nsd)
 	struct gsm_gprs_nse *nse;
 	struct gsm_gprs_cell *cell;
 
-	if (!is_ipaccess_bts(nsd->bts))
+	if (!is_ipa_abisip_bts(nsd->bts))
 		return 0;
 
 	switch (obj_class) {
@@ -202,7 +202,7 @@ static int sw_activ_rep(struct msgb *mb)
 	struct gsm_gprs_nsvc *nsvc;
 	struct gsm_bts_trx_ts *ts;
 
-	if (!is_ipaccess_bts(bts))
+	if (!is_ipa_abisip_bts(bts))
 		return 0;
 
 	switch (foh->obj_class) {
@@ -254,7 +254,6 @@ static void nm_rx_opstart_ack_chan(struct msgb *oml_msg)
 		return;
 	}
 	osmo_fsm_inst_dispatch(ts->mo.fi, NM_EV_OPSTART_ACK, NULL);
-	osmo_fsm_inst_dispatch(ts->fi, TS_EV_OML_READY, NULL);
 }
 
 static void nm_rx_opstart_ack(struct msgb *oml_msg)
@@ -565,7 +564,7 @@ find_bts_by_unitid(struct gsm_network *net, uint16_t site_id, uint16_t bts_id)
 	struct gsm_bts *bts;
 
 	llist_for_each_entry(bts, &net->bts_list, list) {
-		if (!is_ipaccess_bts(bts))
+		if (!is_ipa_abisip_bts(bts))
 			continue;
 
 		if (bts->ip_access.site_id == site_id &&
@@ -637,10 +636,14 @@ void ipaccess_drop_oml(struct gsm_bts *bts, const char *reason)
 	for (i = 0; i < ARRAY_SIZE(bts->site_mgr->gprs.nsvc); i++)
 		osmo_fsm_inst_dispatch(bts->site_mgr->gprs.nsvc[i].mo.fi, NM_EV_OML_DOWN, NULL);
 
-	gsm_bts_all_ts_dispatch(bts, TS_EV_OML_DOWN, NULL);
-
 	bts->ip_access.flags = 0;
 
+	if (bts->model->features_get_reported) {
+		/* Reset the feature vector */
+		memset(bts->_features_data, 0, sizeof(bts->_features_data));
+		bts->features_known = false;
+	}
+
 	/*
 	 * Go through the list and see if we are the depndency of a BTS
 	 * and then drop the BTS. This can lead to some recursion but it
