@@ -1,7 +1,7 @@
 /* Osmocom OsmoBTS specific code */
 
 /* (C) 2010-2012 by Harald Welte <laforge@gnumonks.org>
- * (C) 2021 by sysmocom - s.m.f.c. GmbH <info@sysmocom.de>
+ * (C) 2021 by sysmocom - s.f.m.c. GmbH <info@sysmocom.de>
  *
  * All Rights Reserved
  *
@@ -159,8 +159,7 @@ void osmobts_enc_power_params_osmo_ext(struct msgb *msg, const struct gsm_power_
 	*ie_len = msg->tail - (ie_len + 1);
 }
 
-static int power_ctrl_set_c0_power_red(const struct gsm_bts *bts,
-				       const uint8_t red)
+static int power_ctrl_send_c0_power_red(const struct gsm_bts *bts, const uint8_t red)
 {
 	struct abis_rsl_dchan_hdr *dh;
 	struct msgb *msg;
@@ -169,10 +168,6 @@ static int power_ctrl_set_c0_power_red(const struct gsm_bts *bts,
 	if (msg == NULL)
 		return -ENOMEM;
 
-	LOGP(DRSL, LOGL_NOTICE, "%sabling BCCH carrier power reduction "
-	     "operation mode for BTS%u (maximum %u dB)\n",
-	     red ? "En" : "Dis", bts->nr, red);
-
 	/* Abuse the standard BS POWER CONTROL message by specifying 'Common Channel'
 	 * in the Protocol Discriminator field and 'BCCH' in the Channel Number IE. */
 	dh = (struct abis_rsl_dchan_hdr *) msgb_put(msg, sizeof(*dh));
@@ -193,12 +188,13 @@ int bts_model_osmobts_init(void)
 	model_osmobts = bts_model_nanobts;
 	model_osmobts.name = "osmo-bts";
 	model_osmobts.type = GSM_BTS_TYPE_OSMOBTS;
+	model_osmobts.features_get_reported = true;
 
 	/* Unlike nanoBTS, osmo-bts does support SI2bis and SI2ter fine */
 	model_osmobts.force_combined_si = false;
 
 	/* Power control API */
-	model_osmobts.power_ctrl_set_c0_power_red = &power_ctrl_set_c0_power_red;
+	model_osmobts.power_ctrl_send_c0_power_red = &power_ctrl_send_c0_power_red;
 
 	model_osmobts.features.data = &model_osmobts._features_data[0];
 	model_osmobts.features.data_len =
