@@ -61,8 +61,7 @@ static int full_rate_from_perm_spch(bool * full_rate,
 	return 0;
 }
 
-/* Helper function for match_codec_pref(), looks up a matching chan mode for
- * a given permitted speech value */
+/* Look up a matching chan mode for a given permitted speech value */
 static enum gsm48_chan_mode gsm88_to_chan_mode(enum gsm0808_permitted_speech speech)
 {
 	switch (speech) {
@@ -88,8 +87,7 @@ static enum gsm48_chan_mode gsm88_to_chan_mode(enum gsm0808_permitted_speech spe
 	}
 }
 
-/* Helper function for match_codec_pref(), looks up a matching permitted speech
- * value for a given msc audio codec pref */
+/* Look up a matching permitted speech value for a given msc audio codec pref */
 static enum gsm0808_permitted_speech audio_support_to_gsm88(const struct gsm_audio_support *audio)
 {
 	if (audio->hr) {
@@ -125,10 +123,9 @@ static enum gsm0808_permitted_speech audio_support_to_gsm88(const struct gsm_aud
 	}
 }
 
-/* Helper function for match_codec_pref(), tests if a given audio support
- * matches one of the permitted speech settings of the channel type element.
- * The matched permitted speech value is then also compared against the
- * speech codec list. (optional, only relevant for AoIP) */
+/* Test if a given audio support matches one of the permitted speech settings
+ * of the channel type element. The matched permitted speech value is then also
+ * compared against the speech codec list. (optional, only relevant for AoIP) */
 static bool test_codec_pref(const struct gsm0808_speech_codec **sc_match,
 			    const struct gsm0808_speech_codec_list *scl,
 			    const struct gsm0808_channel_type *ct,
@@ -177,37 +174,48 @@ static bool test_codec_pref(const struct gsm0808_speech_codec **sc_match,
 	return false;
 }
 
-/* Helper function to check if the given permitted speech value is supported
- * by the BTS. (vty option bts->codec-support). */
+static bool test_codec_support_bts_rate(const struct gsm_bts *bts, const bool full_rate)
+{
+	unsigned int i;
+	struct gsm_bts_trx *trx;
+
+	llist_for_each_entry(trx, &bts->trx_list, list) {
+		for (i = 0; i < TRX_NR_TS; i++) {
+			switch (trx->ts[i].pchan_from_config) {
+			case GSM_PCHAN_OSMO_DYN:
+				return true;
+			case GSM_PCHAN_TCH_F:
+			case GSM_PCHAN_TCH_F_PDCH:
+				if (full_rate)
+					return true;
+				break;
+			case GSM_PCHAN_TCH_H:
+				if (!full_rate)
+					return true;
+				break;
+			default:
+				continue;
+			}
+		}
+	}
+
+	return false;
+}
+
+/* Check if the given permitted speech value is supported by the BTS
+ * (vty option bts->codec-support). */
 static bool test_codec_support_bts(const struct gsm_bts *bts, uint8_t perm_spch)
 {
-	struct gsm_bts_trx *trx;
 	const struct bts_codec_conf *bts_codec = &bts->codec;
-	unsigned int i;
 	bool full_rate;
 	int rc;
-	enum gsm_phys_chan_config pchan;
-	bool rate_match = false;
 
 	/* Check if the BTS provides a physical channel that matches the
 	 * bandwidth of the desired codec. */
 	rc = full_rate_from_perm_spch(&full_rate, perm_spch);
 	if (rc < 0)
 		return false;
-	llist_for_each_entry(trx, &bts->trx_list, list) {
-		for (i = 0; i < TRX_NR_TS; i++) {
-			pchan = trx->ts[i].pchan_from_config;
-			if (pchan == GSM_PCHAN_OSMO_DYN)
-				rate_match = true;
-			else if (full_rate && pchan == GSM_PCHAN_TCH_F)
-				rate_match = true;
-			else if (full_rate && pchan == GSM_PCHAN_TCH_F_PDCH)
-				rate_match = true;
-			else if (!full_rate && pchan == GSM_PCHAN_TCH_H)
-				rate_match = true;
-		}
-	}
-	if (!rate_match)
+	if (!test_codec_support_bts_rate(bts, full_rate))
 		return false;
 
 	/* Check codec support */
@@ -217,21 +225,12 @@ static bool test_codec_support_bts(const struct gsm_bts *bts, uint8_t perm_spch)
 		 * selectively disable GSM-RF per BTS via VTY. */
 		return true;
 	case GSM0808_PERM_FR2:
-		if (bts_codec->efr)
-			return true;
-		break;
+		return (bool)bts_codec->efr;
 	case GSM0808_PERM_FR3:
-		if (bts_codec->amr)
-			return true;
-		break;
-	case GSM0808_PERM_HR1:
-		if (bts_codec->hr)
-			return true;
-		break;
 	case GSM0808_PERM_HR3:
-		if (bts_codec->amr)
-			return true;
-		break;
+		return (bool)bts_codec->amr;
+	case GSM0808_PERM_HR1:
+		return (bool)bts_codec->hr;
 	default:
 		return false;
 	}
@@ -308,8 +307,8 @@ static int match_amr_s15_s0(struct channel_mode_and_rate *ch_mode_rate, const st
 	return 0;
 }
 
-/*! Match the codec preferences from local config with a received codec preferences IEs received from the
- * MSC and the BTS' codec configuration.
+/*! Match the codec preferences from local config with codec preference IEs
+ * received from the MSC and the BTS' codec configuration.
  *  \param[out] ch_mode_rate resulting codec and rate information
  *  \param[in] ct GSM 08.08 channel type received from MSC.
  *  \param[in] scl GSM 08.08 speech codec list received from MSC (optional).
@@ -336,7 +335,7 @@ int match_codec_pref(struct channel_mode_and_rate *ch_mode_rate,
 	 * indeed available with the current BTS and MSC configuration */
 	for (i = 0; i < msc->audio_length; i++) {
 		/* Pick a permitted speech value from the global codec configuration list */
-		perm_spch = audio_support_to_gsm88(msc->audio_support[i]);
+		perm_spch = audio_support_to_gsm88(&msc->audio_support[i]);
 
 		/* Determine if the result is a half or full rate codec */
 		rc = full_rate_from_perm_spch(&full_rate, perm_spch);
@@ -376,7 +375,7 @@ int match_codec_pref(struct channel_mode_and_rate *ch_mode_rate,
 		break;
 	}
 
-	/* Exit without result, in case no match can be deteched */
+	/* Exit without result, in case no match can be detected */
 	if (!match) {
 		ch_mode_rate->chan_mode = GSM48_CMODE_SIGN;
 		ch_mode_rate->chan_rate = CH_RATE_SDCCH;
@@ -407,7 +406,7 @@ void gen_bss_supported_codec_list(struct gsm0808_speech_codec_list *scl,
 	for (i = 0; i < msc->audio_length; i++) {
 
 		/* Pick a permitted speech value from the global codec configuration list */
-		perm_spch = audio_support_to_gsm88(msc->audio_support[i]);
+		perm_spch = audio_support_to_gsm88(&msc->audio_support[i]);
 
 		/* Check this permitted speech value against the BTS specific parameters.
 		 * if the BTS does not support the codec, try the next one */
@@ -422,8 +421,8 @@ void gen_bss_supported_codec_list(struct gsm0808_speech_codec_list *scl,
 		/* AMR (HR/FR version 3) is the only codec that requires a codec
 		 * configuration (S0-S15). Determine the current configuration and update
 		 * the cfg flag. */
-		if (msc->audio_support[i]->ver == 3)
-			scl->codec[scl->len].cfg = gen_bss_supported_amr_s15_s0(msc, bts, msc->audio_support[i]->hr);
+		if (msc->audio_support[i].ver == 3)
+			scl->codec[scl->len].cfg = gen_bss_supported_amr_s15_s0(msc, bts, msc->audio_support[i].hr);
 
 		scl->len++;
 	}
