@@ -184,31 +184,22 @@ const struct value_string gsm_pchant_names[] = {
 	{ GSM_PCHAN_TCH_H,	"TCH/H" },
 	{ GSM_PCHAN_SDCCH8_SACCH8C, "SDCCH8" },
 	{ GSM_PCHAN_PDCH,	"PDCH" },
-	{ GSM_PCHAN_TCH_F_PDCH,	"TCH/F_PDCH" },
+	{ GSM_PCHAN_TCH_F_PDCH,	"DYNAMIC/IPACCESS" },
 	{ GSM_PCHAN_UNKNOWN,	"UNKNOWN" },
 	{ GSM_PCHAN_CCCH_SDCCH4_CBCH, "CCCH+SDCCH4+CBCH" },
 	{ GSM_PCHAN_SDCCH8_SACCH8C_CBCH, "SDCCH8+CBCH" },
+	{ GSM_PCHAN_OSMO_DYN, "DYNAMIC/OSMOCOM" },
+	/* make get_string_value() return GSM_PCHAN_TCH_F_PDCH for both "DYNAMIC/IPACCESS" and "TCH/F_PDCH" */
+	{ GSM_PCHAN_TCH_F_PDCH,	"TCH/F_PDCH" },
+	/* make get_string_value() return GSM_PCHAN_OSMO_DYN for both "DYNAMIC/OSMOCOM" and "TCH/F_TCH/H_SDCCH8_PDCH" */
 	{ GSM_PCHAN_OSMO_DYN, "TCH/F_TCH/H_SDCCH8_PDCH" },
+	/* When adding items here, you must also add matching items to gsm_pchant_descs[]! */
 	{ 0,			NULL }
 };
 
-const struct value_string gsm_pchan_ids[] = {
-	{ GSM_PCHAN_NONE,	"NONE" },
-	{ GSM_PCHAN_CCCH,	"CCCH" },
-	{ GSM_PCHAN_CCCH_SDCCH4,"CCCH_SDCCH4" },
-	{ GSM_PCHAN_TCH_F,	"TCH_F" },
-	{ GSM_PCHAN_TCH_H,	"TCH_H" },
-	{ GSM_PCHAN_SDCCH8_SACCH8C, "SDCCH8" },
-	{ GSM_PCHAN_PDCH,	"PDCH" },
-	{ GSM_PCHAN_TCH_F_PDCH,	"TCH_F_PDCH" },
-	{ GSM_PCHAN_UNKNOWN,	"UNKNOWN" },
-	{ GSM_PCHAN_CCCH_SDCCH4_CBCH, "CCCH_SDCCH4_CBCH" },
-	{ GSM_PCHAN_SDCCH8_SACCH8C_CBCH, "SDCCH8_CBCH" },
-	{ GSM_PCHAN_OSMO_DYN, "OSMO_DYN" },
-	{ 0,			NULL }
-};
-
-const struct value_string gsm_pchant_descs[13] = {
+/* VTY command descriptions. These have to be in the same order as gsm_pchant_names[], so that the automatic VTY command
+ * composition in bts_trx_vty_init() works out. */
+const struct value_string gsm_pchant_descs[] = {
 	{ GSM_PCHAN_NONE,	"Physical Channel not configured" },
 	{ GSM_PCHAN_CCCH,	"FCCH + SCH + BCCH + CCCH (Comb. IV)" },
 	{ GSM_PCHAN_CCCH_SDCCH4,
@@ -217,14 +208,24 @@ const struct value_string gsm_pchant_descs[13] = {
 	{ GSM_PCHAN_TCH_H,	"2 TCH/H + 2 FACCH/H + 2 SACCH (Comb. II)" },
 	{ GSM_PCHAN_SDCCH8_SACCH8C, "8 SDCCH + 4 SACCH (Comb. VII)" },
 	{ GSM_PCHAN_PDCH,	"Packet Data Channel for GPRS/EDGE" },
-	{ GSM_PCHAN_TCH_F_PDCH,	"Dynamic TCH/F or GPRS PDCH" },
+	{ GSM_PCHAN_TCH_F_PDCH,	"Dynamic TCH/F or GPRS PDCH"
+				" (dynamic/ipaccess is an alias for tch/f_pdch)" },
 	{ GSM_PCHAN_UNKNOWN,	"Unknown / Unsupported channel combination" },
 	{ GSM_PCHAN_CCCH_SDCCH4_CBCH, "FCCH + SCH + BCCH + CCCH + CBCH + 3 SDCCH + 2 SACCH (Comb. V)" },
 	{ GSM_PCHAN_SDCCH8_SACCH8C_CBCH, "7 SDCCH + 4 SACCH + CBCH (Comb. VII)" },
-	{ GSM_PCHAN_OSMO_DYN, "Dynamic TCH/F or TCH/H or SDCCH/8 or GPRS PDCH" },
+	{ GSM_PCHAN_OSMO_DYN,	"Dynamic TCH/F or TCH/H or SDCCH/8 or GPRS PDCH"
+				" (dynamic/osmocom is an alias for tch/f_tch/h_sdcch8_pdch)" },
+	/* These duplicate entries are needed to provide a description for both the DYNAMIC/... aliases and their
+	 * explicit versions 'TCH/F_PDCH' / 'TCH/F_TCH/H_SDCCH8_PDCH', see bts_trx_vty_init() */
+	{ GSM_PCHAN_TCH_F_PDCH,	"Dynamic TCH/F or GPRS PDCH"
+				" (dynamic/ipaccess is an alias for tch/f_pdch)" },
+	{ GSM_PCHAN_OSMO_DYN,	"Dynamic TCH/F or TCH/H or SDCCH/8 or GPRS PDCH"
+				" (dynamic/osmocom is an alias for tch/f_tch/h_sdcch8_pdch)" },
 	{ 0,			NULL }
 };
 
+osmo_static_assert(ARRAY_SIZE(gsm_pchant_names) == ARRAY_SIZE(gsm_pchant_descs), _pchan_vty_docs);
+
 const char *gsm_pchan_name(enum gsm_phys_chan_config c)
 {
 	return get_value_string(gsm_pchant_names, c);
@@ -939,6 +940,7 @@ const struct value_string lchan_activate_mode_names[] = {
 	OSMO_VALUE_STRING(ACTIVATE_FOR_MS_CHANNEL_REQUEST),
 	OSMO_VALUE_STRING(ACTIVATE_FOR_ASSIGNMENT),
 	OSMO_VALUE_STRING(ACTIVATE_FOR_HANDOVER),
+	OSMO_VALUE_STRING(ACTIVATE_FOR_VGCS_CHANNEL),
 	OSMO_VALUE_STRING(ACTIVATE_FOR_VTY),
 	{}
 };
@@ -1042,3 +1044,18 @@ enum rsl_cmod_spd chan_mode_to_rsl_cmod_spd(enum gsm48_chan_mode chan_mode)
 		return -EINVAL;
 	}
 }
+
+int gsm_audio_support_cmp(const struct gsm_audio_support *a, const struct gsm_audio_support *b)
+{
+	int rc;
+	if (a == b)
+		return 0;
+	if (!a)
+		return -1;
+	if (!b)
+		return 1;
+	rc = OSMO_CMP(a->hr, b->hr);
+	if (rc)
+		return rc;
+	return OSMO_CMP(a->ver, b->ver);
+}
