@@ -53,7 +53,7 @@
 	     lchan->ts->trx->nr, \
 	     lchan->ts->nr, \
 	     lchan->nr, \
-	     gsm_lchant_name(lchan->type), \
+	     gsm_chan_t_name(lchan->type), \
 	     gsm48_chan_mode_name(lchan->current_ch_mode_rate.chan_mode), \
 	     bsc_subscr_name(lchan->conn? lchan->conn->bsub : NULL), \
 	     ## args)
@@ -64,7 +64,7 @@
 	     lchan->ts->trx->nr, \
 	     lchan->ts->nr, \
 	     lchan->nr, \
-	     gsm_lchant_name(lchan->type), \
+	     gsm_chan_t_name(lchan->type), \
 	     gsm48_chan_mode_name(lchan->current_ch_mode_rate.chan_mode), \
 	     new_bts->nr, \
 	     bsc_subscr_name(lchan->conn? lchan->conn->bsub : NULL), \
@@ -76,7 +76,7 @@
 	     lchan->ts->trx->nr, \
 	     lchan->ts->nr, \
 	     lchan->nr, \
-	     gsm_lchant_name(lchan->type), \
+	     gsm_chan_t_name(lchan->type), \
 	     gsm48_chan_mode_name(lchan->current_ch_mode_rate.chan_mode), \
 	     gsm0808_cell_id_list_name(remote_cil), \
 	     bsc_subscr_name(lchan->conn? lchan->conn->bsub : NULL), \
@@ -519,7 +519,7 @@ static void check_requirements(struct ho_candidate *c)
 						 "tch_mode='%s' type='%s' not supported\n",
 						 get_value_string(gsm48_chan_mode_names,
 								  c->current.lchan->current_ch_mode_rate.chan_mode),
-						 gsm_lchant_name(c->current.lchan->type));
+						 gsm_chan_t_name(c->current.lchan->type));
 				break;
 			}
 			if (codec_type_is_supported(c->current.lchan->conn, GSM0808_SCT_HR1))
@@ -914,6 +914,19 @@ static int trigger_local_ho_or_as(struct ho_candidate *c, uint8_t requirements)
 				 full_rate ? "TCH/F" : "TCH/H",
 				 ho_reason_name(global_ho_reason));
 		handover_request(&req);
+
+		/* Apply penalty timer hodec2_penalty_low_rxqual_ho */
+		if (global_ho_reason == HO_REASON_INTERFERENCE
+		    || global_ho_reason == HO_REASON_BAD_QUALITY) {
+			struct gsm0808_cell_id bts_id;
+			struct gsm_subscriber_connection *conn = c->current.lchan->conn;
+			int timeout = ho_get_hodec2_penalty_low_rxqual_ho(c->current.bts->ho);
+			gsm_bts_cell_id(&bts_id, c->current.bts);
+			LOGPHOCAND(c, LOGL_DEBUG, "Applying penalty-time low-rxqual-ho %d s on bts %u (%s), reason: %s\n",
+				   timeout, c->current.bts->nr, gsm0808_cell_id_name_c(OTC_SELECT, &bts_id),
+				   ho_reason_name(global_ho_reason));
+			penalty_timers_add(conn, &conn->hodec2.penalty_timers, &bts_id, timeout);
+		}
 	}
 	return 0;
 }
@@ -991,13 +1004,13 @@ static inline void debug_candidate(struct ho_candidate *candidate)
 
 static void candidate_set_free_tch(struct ho_candidate *c)
 {
-	struct chan_counts bts_counts;
+	struct chan_counts *bts_counts;
 	struct gsm_lchan *next_lchan;
 
-	chan_counts_for_bts(&bts_counts, c->current.bts);
-	c->current.free_tchf = bts_counts.val[CHAN_COUNTS1_ALL][CHAN_COUNTS2_FREE][GSM_LCHAN_TCH_F];
+	bts_counts = &c->current.bts->chan_counts;
+	c->current.free_tchf = bts_counts->val[CHAN_COUNTS1_ALL][CHAN_COUNTS2_FREE][GSM_LCHAN_TCH_F];
 	c->current.min_free_tchf = ho_get_hodec2_tchf_min_slots(c->current.bts->ho);
-	c->current.free_tchh = bts_counts.val[CHAN_COUNTS1_ALL][CHAN_COUNTS2_FREE][GSM_LCHAN_TCH_H];
+	c->current.free_tchh = bts_counts->val[CHAN_COUNTS1_ALL][CHAN_COUNTS2_FREE][GSM_LCHAN_TCH_H];
 	c->current.min_free_tchh = ho_get_hodec2_tchh_min_slots(c->current.bts->ho);
 
 	switch (c->current.lchan->ts->pchan_is) {
@@ -1029,15 +1042,16 @@ static void candidate_set_free_tch(struct ho_candidate *c)
 
 	/* For inter-BSC handover, the target BTS is in a different BSC and hence NULL here. */
 	if (c->target.bts) {
-		chan_counts_for_bts(&bts_counts, c->target.bts);
-		c->target.free_tchf = bts_counts.val[CHAN_COUNTS1_ALL][CHAN_COUNTS2_FREE][GSM_LCHAN_TCH_F];
+		bts_counts = &c->target.bts->chan_counts;
+		c->target.free_tchf = bts_counts->val[CHAN_COUNTS1_ALL][CHAN_COUNTS2_FREE][GSM_LCHAN_TCH_F];
 		c->target.min_free_tchf = ho_get_hodec2_tchf_min_slots(c->target.bts->ho);
-		c->target.free_tchh = bts_counts.val[CHAN_COUNTS1_ALL][CHAN_COUNTS2_FREE][GSM_LCHAN_TCH_H];
+		c->target.free_tchh = bts_counts->val[CHAN_COUNTS1_ALL][CHAN_COUNTS2_FREE][GSM_LCHAN_TCH_H];
 		c->target.min_free_tchh = ho_get_hodec2_tchh_min_slots(c->target.bts->ho);
 
 		/* Would the next TCH/F lchan occupy a dynamic timeslot that currently counts for free TCH/H timeslots?
 		 */
-		next_lchan = lchan_avail_by_type(c->target.bts, GSM_LCHAN_TCH_F, false);
+		next_lchan = lchan_avail_by_type(c->target.bts, GSM_LCHAN_TCH_F,
+						 SELECT_FOR_HANDOVER, NULL, false);
 		if (next_lchan && next_lchan->ts->pchan_on_init == GSM_PCHAN_OSMO_DYN)
 			c->target.next_tchf_reduces_tchh = 2;
 		else
@@ -1045,7 +1059,8 @@ static void candidate_set_free_tch(struct ho_candidate *c)
 
 		/* Would the next TCH/H lchan occupy a dynamic timeslot that currently counts for free TCH/F timeslots?
 		 * Note that a dyn TS already in TCH/H mode (half occupied) would not reduce free TCH/F. */
-		next_lchan = lchan_avail_by_type(c->target.bts, GSM_LCHAN_TCH_H, false);
+		next_lchan = lchan_avail_by_type(c->target.bts, GSM_LCHAN_TCH_H,
+						 SELECT_FOR_HANDOVER, NULL, false);
 		if (next_lchan && next_lchan->ts->pchan_on_init == GSM_PCHAN_OSMO_DYN
 		    && next_lchan->ts->pchan_is != GSM_PCHAN_TCH_H)
 			c->target.next_tchh_reduces_tchf = 1;
@@ -1435,7 +1450,8 @@ static int find_alternative_lchan(struct gsm_lchan *lchan, bool include_weaker_r
 
 		/* After upgrading TCH/H to TCH/F due to bad RxQual, start penalty timer to avoid re-assignment within
 		 * the same cell again, to avoid oscillation from RxQual noise combined with congestion resolution. */
-		if (!rc && is_upgrade_to_tchf(best_cand, REQUIREMENT_A_MASK)) {
+		if (!rc && best_cand->target.bts == best_cand->current.bts
+		    && is_upgrade_to_tchf(best_cand, REQUIREMENT_A_MASK)) {
 			struct gsm0808_cell_id bts_id;
 			gsm_bts_cell_id(&bts_id, best_cand->target.bts);
 			penalty_timers_add(lchan->conn, &lchan->conn->hodec2.penalty_timers, &bts_id,
@@ -1945,7 +1961,7 @@ exit:
 
 static void bts_congestion_check(struct gsm_bts *bts)
 {
-	struct chan_counts bts_counts;
+	struct chan_counts *bts_counts;
 	int min_free_tchf, min_free_tchh;
 	int free_tchf, free_tchh;
 
@@ -1973,9 +1989,9 @@ static void bts_congestion_check(struct gsm_bts *bts)
 		return;
 	}
 
-	chan_counts_for_bts(&bts_counts, bts);
-	free_tchf = bts_counts.val[CHAN_COUNTS1_ALL][CHAN_COUNTS2_FREE][GSM_LCHAN_TCH_F];
-	free_tchh = bts_counts.val[CHAN_COUNTS1_ALL][CHAN_COUNTS2_FREE][GSM_LCHAN_TCH_H];
+	bts_counts = &bts->chan_counts;
+	free_tchf = bts_counts->val[CHAN_COUNTS1_ALL][CHAN_COUNTS2_FREE][GSM_LCHAN_TCH_F];
+	free_tchh = bts_counts->val[CHAN_COUNTS1_ALL][CHAN_COUNTS2_FREE][GSM_LCHAN_TCH_H];
 	LOGPHOBTS(bts, LOGL_INFO, "Congestion check: (free/want-free) TCH/F=%d/%d TCH/H=%d/%d\n",
 		  free_tchf, min_free_tchf, free_tchh, min_free_tchh);
 
