@@ -31,19 +31,7 @@
 #include <osmocom/bsc/osmo_bsc_sigtran.h>
 #include <osmocom/bsc/lcs_loc_req.h>
 #include <osmocom/bsc/bssmap_reset.h>
-
-static struct gsm_subscriber_connection *get_bsc_conn_by_lb_conn_id(int conn_id)
-{
-	struct gsm_subscriber_connection *conn;
-
-	llist_for_each_entry(conn, &bsc_gsmnet->subscr_conns, entry) {
-		if (conn->lcs.lb.state != SUBSCR_SCCP_ST_NONE
-		    && conn->lcs.lb.conn_id == conn_id)
-			return conn;
-	}
-
-	return NULL;
-}
+#include <osmocom/bsc/gsm_data.h>
 
 /* Send reset to SMLC */
 int bssmap_le_tx_reset(void)
@@ -147,6 +135,8 @@ static int sccp_sap_up(struct osmo_prim_hdr *oph, void *_scu)
 {
 	struct osmo_scu_prim *scu_prim = (struct osmo_scu_prim *)oph;
 	struct osmo_sccp_user *scu = _scu;
+	struct osmo_sccp_instance *sccp = osmo_sccp_get_sccp(scu);
+	struct bsc_sccp_inst *bsc_sccp = osmo_sccp_get_priv(sccp);
 	struct gsm_subscriber_connection *conn;
 	int rc = 0;
 
@@ -169,7 +159,7 @@ static int sccp_sap_up(struct osmo_prim_hdr *oph, void *_scu)
 	case OSMO_PRIM(OSMO_SCU_PRIM_N_CONNECT, PRIM_OP_CONFIRM):
 		/* Handle inbound confirmation of outbound connection */
 		DEBUGP(DLCS, "N-CONNECT.cnf(%u)\n", scu_prim->u.connect.conn_id);
-		conn = get_bsc_conn_by_lb_conn_id(scu_prim->u.connect.conn_id);
+		conn = bsc_sccp_inst_get_gscon_by_conn_id(bsc_sccp, scu_prim->u.connect.conn_id);
 		if (conn) {
 			conn->lcs.lb.state = SUBSCR_SCCP_ST_CONNECTED;
 			if (msgb_l2len(oph->msg) > 0) {
@@ -185,7 +175,7 @@ static int sccp_sap_up(struct osmo_prim_hdr *oph, void *_scu)
 		/* Handle incoming connection oriented data */
 		DEBUGP(DLCS, "N-DATA.ind(%u)\n", scu_prim->u.data.conn_id);
 
-		conn = get_bsc_conn_by_lb_conn_id(scu_prim->u.data.conn_id);
+		conn = bsc_sccp_inst_get_gscon_by_conn_id(bsc_sccp, scu_prim->u.data.conn_id);
 		if (!conn) {
 			LOGP(DLCS, LOGL_ERROR, "N-DATA.ind(%u) for unknown conn_id\n", scu_prim->u.data.conn_id);
 			rc = -EINVAL;
@@ -203,12 +193,14 @@ static int sccp_sap_up(struct osmo_prim_hdr *oph, void *_scu)
 		       osmo_hexdump(msgb_l2(oph->msg), msgb_l2len(oph->msg)),
 		       scu_prim->u.disconnect.cause);
 		/* indication of disconnect */
-		conn = get_bsc_conn_by_lb_conn_id(scu_prim->u.disconnect.conn_id);
+		conn = bsc_sccp_inst_get_gscon_by_conn_id(bsc_sccp, scu_prim->u.disconnect.conn_id);
 		if (!conn) {
 			LOGP(DLCS, LOGL_ERROR, "N-DISCONNECT.ind for unknown conn_id %u\n",
 			     scu_prim->u.disconnect.conn_id);
 			rc = -EINVAL;
 		} else {
+			bsc_sccp_inst_unregister_gscon(bsc_sccp, &conn->lcs.lb.conn);
+			conn->lcs.lb.conn.conn_id = SCCP_CONN_ID_UNSET;
 			conn->lcs.lb.state = SUBSCR_SCCP_ST_NONE;
 			if (msgb_l2len(oph->msg) > 0) {
 				rc = lcs_loc_req_rx_bssmap_le(conn, oph->msg);
@@ -229,8 +221,9 @@ static int sccp_sap_up(struct osmo_prim_hdr *oph, void *_scu)
 static int lb_open_conn(struct gsm_subscriber_connection *conn, struct msgb *msg)
 {
 	struct osmo_ss7_instance *ss7;
-	int conn_id;
+	uint32_t conn_id;
 	int rc;
+	struct bsc_sccp_inst *bsc_sccp = osmo_sccp_get_priv(bsc_gsmnet->smlc->sccp);
 
 	OSMO_ASSERT(conn);
 	OSMO_ASSERT(msg);
@@ -241,35 +234,52 @@ static int lb_open_conn(struct gsm_subscriber_connection *conn, struct msgb *msg
 		return -EINVAL;
 	}
 
-	conn_id = bsc_sccp_inst_next_conn_id(bsc_gsmnet->smlc->sccp);
-	if (conn_id < 0) {
+	conn_id = bsc_sccp_inst_next_conn_id(bsc_sccp);
+	if (conn_id == SCCP_CONN_ID_UNSET) {
 		LOGPFSMSL(conn->fi, DLCS, LOGL_ERROR, "Unable to allocate SCCP Connection ID for BSSMAP-LE to SMLC\n");
 		return -ENOSPC;
 	}
-	conn->lcs.lb.conn_id = conn_id;
+	conn->lcs.lb.conn.conn_id = conn_id;
+	if (bsc_sccp_inst_register_gscon(bsc_sccp, &conn->lcs.lb.conn) < 0) {
+		LOGP(DMSC, LOGL_ERROR, "Unable to register Lb SCCP connection (id=%u)\n", conn->lcs.lb.conn.conn_id);
+		return -1;
+	}
 	ss7 = osmo_ss7_instance_find(bsc_gsmnet->smlc->cs7_instance);
 	OSMO_ASSERT(ss7);
-	LOGPFSMSL(conn->fi, DLCS, LOGL_INFO, "Opening new SCCP connection (id=%i) to SMLC: %s\n", conn_id,
+	LOGPFSMSL(conn->fi, DLCS, LOGL_INFO, "Opening new SCCP connection (id=%u) to SMLC: %s\n", conn_id,
 		  osmo_sccp_addr_name(ss7, &bsc_gsmnet->smlc->smlc_addr));
 
 	rc = osmo_sccp_tx_conn_req_msg(bsc_gsmnet->smlc->sccp_user, conn_id, &bsc_gsmnet->smlc->bsc_addr,
 				       &bsc_gsmnet->smlc->smlc_addr, msg);
-	if (rc >= 0)
+	if (rc >= 0) {
 		rate_ctr_inc(rate_ctr_group_get_ctr(bsc_gsmnet->smlc->ctrs, SMLC_CTR_BSSMAP_LE_TX_SUCCESS));
-	else
-		rate_ctr_inc(rate_ctr_group_get_ctr(bsc_gsmnet->smlc->ctrs, SMLC_CTR_BSSMAP_LE_TX_ERR_SEND));
-	if (rc >= 0)
 		conn->lcs.lb.state = SUBSCR_SCCP_ST_WAIT_CONN_CONF;
+	} else {
+		rate_ctr_inc(rate_ctr_group_get_ctr(bsc_gsmnet->smlc->ctrs, SMLC_CTR_BSSMAP_LE_TX_ERR_SEND));
+		goto failed_unregister_conn_id;
+	}
 
 	return rc;
+
+failed_unregister_conn_id:
+	bsc_sccp_inst_unregister_gscon(bsc_sccp, &conn->lcs.lb.conn);
+	conn->lcs.lb.conn.conn_id = SCCP_CONN_ID_UNSET;
+	return rc;
 }
 
 void lb_close_conn(struct gsm_subscriber_connection *conn)
 {
+	struct bsc_sccp_inst *bsc_sccp;
 	if (conn->lcs.lb.state == SUBSCR_SCCP_ST_NONE)
 		return;
-	osmo_sccp_tx_disconn(bsc_gsmnet->smlc->sccp_user, conn->lcs.lb.conn_id, &bsc_gsmnet->smlc->bsc_addr, 0);
+	OSMO_ASSERT(conn->lcs.lb.conn.conn_id != SCCP_CONN_ID_UNSET);
+
+	osmo_sccp_tx_disconn(bsc_gsmnet->smlc->sccp_user, conn->lcs.lb.conn.conn_id, &bsc_gsmnet->smlc->bsc_addr, 0);
+
 	conn->lcs.lb.state = SUBSCR_SCCP_ST_NONE;
+	bsc_sccp = osmo_sccp_get_priv(bsc_gsmnet->smlc->sccp);
+	bsc_sccp_inst_unregister_gscon(bsc_sccp, &conn->lcs.lb.conn);
+	conn->lcs.lb.conn.conn_id = SCCP_CONN_ID_UNSET;
 }
 
 /* Send data to SMLC, take ownership of *msg */
@@ -302,7 +312,7 @@ int lb_send(struct gsm_subscriber_connection *conn, const struct bssap_le_pdu *b
 	}
 
 	LOGPFSMSL(conn->fi, DLCS, LOGL_DEBUG, "Tx %s\n", osmo_bssap_le_pdu_to_str_c(OTC_SELECT, bssap_le));
-	rc = osmo_sccp_tx_data_msg(bsc_gsmnet->smlc->sccp_user, conn->lcs.lb.conn_id, msg);
+	rc = osmo_sccp_tx_data_msg(bsc_gsmnet->smlc->sccp_user, conn->lcs.lb.conn.conn_id, msg);
 	if (rc >= 0)
 		rate_ctr_inc(rate_ctr_group_get_ctr(bsc_gsmnet->smlc->ctrs, SMLC_CTR_BSSMAP_LE_TX_SUCCESS));
 	else
@@ -418,6 +428,7 @@ static int lb_start(void)
 	enum osmo_ss7_asp_protocol used_proto = OSMO_SS7_ASP_PROT_M3UA;
 	char inst_name[32];
 	const char *smlc_name = "smlc";
+	struct bsc_sccp_inst *bsc_sccp;
 
 	/* Already set up? */
 	if (bsc_gsmnet->smlc->sccp_user)
@@ -451,6 +462,9 @@ static int lb_start(void)
 	if (!sccp)
 		return -EINVAL;
 	bsc_gsmnet->smlc->sccp = sccp;
+	bsc_sccp = bsc_sccp_inst_alloc(tall_bsc_ctx);
+	bsc_sccp->sccp = sccp;
+	osmo_sccp_set_priv(sccp, bsc_sccp);
 
 	/* If unset, use default local SCCP address */
 	if (!bsc_gsmnet->smlc->bsc_addr.presence)
