@@ -35,6 +35,7 @@
 #include <osmocom/bsc/abis_rsl.h>
 #include <osmocom/bsc/bsc_rll.h>
 #include <osmocom/bsc/gsm_04_08_rr.h>
+#include <osmocom/bsc/gsm_08_08.h>
 #include <osmocom/bsc/assignment_fsm.h>
 #include <osmocom/bsc/handover_fsm.h>
 #include <osmocom/bsc/bsc_msc_data.h>
@@ -42,6 +43,7 @@
 #include <osmocom/bsc/bts.h>
 #include <osmocom/bsc/bsc_stats.h>
 #include <osmocom/bsc/lchan.h>
+#include <osmocom/bsc/vgcs_fsm.h>
 
 static struct osmo_fsm lchan_fsm;
 
@@ -68,6 +70,14 @@ bool lchan_may_receive_data(struct gsm_lchan *lchan)
 	}
 }
 
+bool lchan_is_asci(struct gsm_lchan *lchan)
+{
+	if (lchan->activate.info.type_for == LCHAN_TYPE_FOR_VGCS ||
+	    lchan->activate.info.type_for == LCHAN_TYPE_FOR_VBS)
+		return true;
+	return false;
+}
+
 static void lchan_on_mode_modify_success(struct gsm_lchan *lchan)
 {
 	lchan->modify.concluded = true;
@@ -183,6 +193,23 @@ static void _lchan_on_activation_failure(struct gsm_lchan *lchan, enum lchan_act
 		_osmo_fsm_inst_dispatch(for_conn->ho.fi, HO_EV_LCHAN_ERROR, lchan, file, line);
 		break;
 
+	case ACTIVATE_FOR_VGCS_CHANNEL:
+		LOG_LCHAN(lchan, LOGL_NOTICE, "Signalling VGCS Assignment FSM of error (%s)\n",
+			  lchan->last_error ? : "unknown error");
+		if (!for_conn) {
+			LOG_LCHAN(lchan, LOGL_ERROR,
+				  "lchan activation for VGCS Assignment failed, but activation request has no conn\n");
+			break;
+		}
+		if (!for_conn->vgcs_chan.fi) {
+			LOG_LCHAN(lchan, LOGL_ERROR,
+				  "lchan activation for VGCS Assignment failed, but conn has no ongoing"
+				  " assignment procedure\n");
+			break;
+		}
+		_osmo_fsm_inst_dispatch(for_conn->vgcs_chan.fi, VGCS_EV_LCHAN_ERROR, lchan, file, line);
+		break;
+
 	case ACTIVATE_FOR_VTY:
 		LOG_LCHAN(lchan, LOGL_ERROR, "VTY user invoked lchan activation failed (%s)\n",
 			  lchan->last_error ? : "unknown error");
@@ -270,6 +297,22 @@ static void lchan_on_fully_established(struct gsm_lchan *lchan)
 		 * we will try to roll back a modified RTP connection. */
 		break;
 
+	case ACTIVATE_FOR_VGCS_CHANNEL:
+		if (!lchan->conn) {
+			LOG_LCHAN(lchan, LOGL_ERROR,
+				  "lchan activation for VGCS assignment succeeded, but lchan has no conn\n");
+			lchan_release(lchan, false, true, RSL_ERR_EQUIPMENT_FAIL, NULL);
+			break;
+		}
+		if (!lchan->conn->vgcs_chan.fi) {
+			LOG_LCHAN(lchan, LOGL_ERROR,
+				  "lchan Activation for VGCS assignment requested, but conn has no VGCS resource FSM.\n");
+			lchan_release(lchan, false, true, RSL_ERR_EQUIPMENT_FAIL, NULL);
+			break;
+		}
+		osmo_fsm_inst_dispatch(lchan->conn->vgcs_chan.fi, VGCS_EV_LCHAN_ACTIVE, lchan);
+		break;
+
 	case ACTIVATE_FOR_MODE_MODIFY_RTP:
 		lchan_on_mode_modify_success(lchan);
 		break;
@@ -347,7 +390,7 @@ void lchan_activate(struct gsm_lchan *lchan, struct lchan_activate_info *info)
 
 	OSMO_ASSERT(lchan && info);
 
-	if ((info->vamos || lchan->vamos.is_secondary)
+	if ((info->type_for == LCHAN_TYPE_FOR_VAMOS || lchan->vamos.is_secondary)
 	    && !osmo_bts_has_feature(&lchan->ts->trx->bts->features, BTS_FEAT_VAMOS)) {
 		lchan->last_error = talloc_strdup(lchan->ts->trx, "VAMOS related channel activation requested,"
 						  " but BTS does not support VAMOS");
@@ -404,6 +447,28 @@ void lchan_activate(struct gsm_lchan *lchan, struct lchan_activate_info *info)
 		}
 		break;
 
+	case ACTIVATE_FOR_VGCS_CHANNEL:
+		if (!info->for_conn
+		    || !info->for_conn->fi) {
+			LOG_LCHAN(lchan, LOGL_ERROR, "Activation requested, but no conn\n");
+			goto abort;
+		}
+		if (!info->for_conn->vgcs_chan.fi)  {
+			LOG_LCHAN(lchan, LOGL_ERROR,
+				  "Activation for VGCS assignment requested, but conn has no VGCS resource FSM.\n");
+			goto abort;
+		}
+		if (info->for_conn->vgcs_chan.new_lchan != lchan) {
+			LOG_LCHAN(lchan, LOGL_ERROR,
+				  "Activation for VGCS assignment requested, but conn's VGCS assignment state does"
+				  " not reflect this lchan to be activated (instead: %s)\n",
+				  info->for_conn->vgcs_chan.new_lchan ?
+					gsm_lchan_name(info->for_conn->vgcs_chan.new_lchan)
+					: "NULL");
+			goto abort;
+		}
+		break;
+
 	default:
 		break;
 	}
@@ -428,7 +493,7 @@ void lchan_mode_modify(struct gsm_lchan *lchan, struct lchan_modify_info *info)
 {
 	OSMO_ASSERT(lchan && info);
 
-	if ((info->vamos || lchan->vamos.is_secondary)
+	if ((info->type_for == LCHAN_TYPE_FOR_VAMOS || lchan->vamos.is_secondary)
 	    && !osmo_bts_has_feature(&lchan->ts->trx->bts->features, BTS_FEAT_VAMOS)) {
 		lchan->last_error = talloc_strdup(lchan->ts->trx, "VAMOS related Channel Mode Modify requested,"
 						  " but BTS does not support VAMOS");
@@ -452,21 +517,18 @@ void lchan_fsm_update_id(struct gsm_lchan *lchan)
 	lchan_update_name(lchan);
 	if (!lchan->fi)
 		return;
-	osmo_fsm_inst_update_id_f(lchan->fi, "%u-%u-%u-%s-%s%u",
-				  lchan->ts->trx->bts->nr, lchan->ts->trx->nr, lchan->ts->nr,
-				  gsm_pchan_id(lchan->ts->pchan_on_init),
-				  lchan->vamos.is_secondary ? "shadow" : "",
-				  lchan->nr - (lchan->vamos.is_secondary ? lchan->ts->max_primary_lchans : 0));
+	osmo_fsm_inst_update_id_f_sanitize(lchan->fi, '_', "%u-%u-%u-%s-%s%u",
+					   lchan->ts->trx->bts->nr, lchan->ts->trx->nr, lchan->ts->nr,
+					   gsm_pchan_name(lchan->ts->pchan_on_init),
+					   lchan->vamos.is_secondary ? "shadow" : "",
+					   lchan->nr - (lchan->vamos.is_secondary ? lchan->ts->max_primary_lchans : 0));
 	if (lchan->fi_rtp)
 		osmo_fsm_inst_update_id_f(lchan->fi_rtp, lchan->fi->id);
 }
 
-extern void lchan_rtp_fsm_init();
-
-void lchan_fsm_init(void)
+static __attribute__((constructor)) void lchan_fsm_init(void)
 {
 	OSMO_ASSERT(osmo_fsm_register(&lchan_fsm) == 0);
-	lchan_rtp_fsm_init();
 }
 
 static void lchan_reset(struct gsm_lchan *lchan);
@@ -678,9 +740,9 @@ static int lchan_activate_set_ch_mode_rate_and_mr_config(struct gsm_lchan *lchan
 {
 	struct osmo_fsm_inst *fi = lchan->fi;
 	lchan->activate.ch_mode_rate = lchan->activate.info.ch_mode_rate;
-	lchan->activate.ch_mode_rate.chan_mode = (lchan->activate.info.vamos
+	lchan->activate.ch_mode_rate.chan_mode = (lchan->activate.info.type_for == LCHAN_TYPE_FOR_VAMOS)
 		? gsm48_chan_mode_to_vamos(lchan->activate.info.ch_mode_rate.chan_mode)
-		: gsm48_chan_mode_to_non_vamos(lchan->activate.info.ch_mode_rate.chan_mode));
+		: gsm48_chan_mode_to_non_vamos(lchan->activate.info.ch_mode_rate.chan_mode);
 	if (lchan->activate.ch_mode_rate.chan_mode < 0) {
 		lchan_fail("Invalid chan_mode: %s", gsm48_chan_mode_name(lchan->activate.info.ch_mode_rate.chan_mode));
 		return -EINVAL;
@@ -692,6 +754,8 @@ static int lchan_activate_set_ch_mode_rate_and_mr_config(struct gsm_lchan *lchan
 			return -EINVAL;
 		}
 	}
+
+	lchan->activate.ch_indctr = lchan->activate.info.ch_indctr;
 	return 0;
 }
 
@@ -720,6 +784,7 @@ static void lchan_fsm_wait_ts_ready_onenter(struct osmo_fsm_inst *fi, uint32_t p
 	struct gsm_lchan *old_lchan = lchan->activate.info.re_use_mgw_endpoint_from_lchan;
 	struct lchan_activate_info *info = &lchan->activate.info;
 	int ms_power_dbm = bts->ms_max_power;
+	bool requires_rtp_stream;
 
 	if (lchan->release.requested) {
 		lchan_fail("Release requested while activating");
@@ -763,12 +828,13 @@ static void lchan_fsm_wait_ts_ready_onenter(struct osmo_fsm_inst *fi, uint32_t p
 	lchan->activate.tsc = lchan->activate.info.tsc.present ? lchan->activate.info.tsc.val : gsm_ts_tsc(lchan->ts);
 
 	use_mgwep_ci = lchan_use_mgw_endpoint_ci_bts(lchan);
+	requires_rtp_stream = bsc_chan_ind_requires_rtp_stream(lchan->activate.info.ch_indctr);
 
 	LOG_LCHAN(lchan, LOGL_INFO,
-		  "Activation requested: %s voice=%s MGW-ci=%s type=%s tch-mode=%s encr-alg=A5/%u ck=%s\n",
+		  "Activation requested: %s rtp=%s MGW-ci=%s type=%s tch-mode=%s encr-alg=A5/%u ck=%s\n",
 		  lchan_activate_mode_name(lchan->activate.info.activ_for),
-		  lchan->activate.info.requires_voice_stream ? "yes" : "no",
-		  lchan->activate.info.requires_voice_stream ?
+		  requires_rtp_stream ? "yes" : "no",
+		  requires_rtp_stream ?
 			(use_mgwep_ci ? osmo_mgcpc_ep_ci_name(use_mgwep_ci) : "new")
 			: "none",
 		  gsm_chan_t_name(lchan->type),
@@ -782,7 +848,7 @@ static void lchan_fsm_wait_ts_ready_onenter(struct osmo_fsm_inst *fi, uint32_t p
 	osmo_fsm_inst_dispatch(lchan->ts->fi, TS_EV_LCHAN_REQUESTED, lchan);
 
 	/* Prepare an MGW endpoint CI if appropriate. */
-	if (lchan->activate.info.requires_voice_stream)
+	if (requires_rtp_stream)
 		lchan_rtp_fsm_start(lchan);
 
 	if (lchan->activate.info.imm_ass_time == IMM_ASS_TIME_PRE_TS_ACK) {
@@ -847,6 +913,7 @@ static void lchan_fsm_wait_activ_ack_onenter(struct osmo_fsm_inst *fi, uint32_t
 		ho_ref = lchan->conn->ho.ho_ref;
 		break;
 	case ACTIVATE_FOR_ASSIGNMENT:
+	case ACTIVATE_FOR_VGCS_CHANNEL:
 	default:
 		act_type = RSL_ACT_INTRA_NORM_ASS;
 		break;
@@ -941,7 +1008,8 @@ static void post_activ_ack_accept_preliminary_settings(struct gsm_lchan *lchan)
 {
 	lchan->current_ch_mode_rate = lchan->activate.ch_mode_rate;
 	lchan->current_mr_conf = lchan->activate.mr_conf_filtered;
-	lchan->vamos.enabled = lchan->activate.info.vamos;
+	lchan->current_ch_indctr = lchan->activate.ch_indctr;
+	lchan->vamos.enabled = (lchan->activate.info.type_for == LCHAN_TYPE_FOR_VAMOS);
 	lchan->tsc_set = lchan->activate.tsc_set;
 	lchan->tsc = lchan->activate.tsc;
 }
@@ -1022,16 +1090,19 @@ static void lchan_fsm_post_activ_ack(struct osmo_fsm_inst *fi)
 static void lchan_fsm_wait_rll_rtp_establish_onenter(struct osmo_fsm_inst *fi, uint32_t prev_state)
 {
 	struct gsm_lchan *lchan = lchan_fi_lchan(fi);
+	bool requires_rtp_stream = bsc_chan_ind_requires_rtp_stream(lchan->activate.info.ch_indctr);
+
 	if (lchan->fi_rtp)
 		osmo_fsm_inst_dispatch(lchan->fi_rtp, LCHAN_RTP_EV_LCHAN_READY, 0);
 	/* Prepare an MGW endpoint CI if appropriate (late). */
-	else if (lchan->activate.info.requires_voice_stream)
+	else if (requires_rtp_stream)
 		lchan_rtp_fsm_start(lchan);
 
-	/* When activating a channel for VTY, skip waiting for activity from
-	 * lchan_rtp_fsm, but only if no voice stream is required. */
-	if (lchan->activate.info.activ_for == ACTIVATE_FOR_VTY &&
-	    !lchan->activate.info.requires_voice_stream) {
+	/* When activating a channel for VTY or VGCS/VBS, skip waiting for activity from
+	 * lchan_rtp_fsm, but only if no rtp stream is required. */
+	if ((lchan->activate.info.activ_for == ACTIVATE_FOR_VTY ||
+	     lchan->activate.info.activ_for == ACTIVATE_FOR_VGCS_CHANNEL) &&
+	    !requires_rtp_stream) {
 		lchan_fsm_state_chg(LCHAN_ST_ESTABLISHED);
 	}
 }
@@ -1039,21 +1110,24 @@ static void lchan_fsm_wait_rll_rtp_establish_onenter(struct osmo_fsm_inst *fi, u
 static void lchan_fsm_wait_rll_rtp_establish(struct osmo_fsm_inst *fi, uint32_t event, void *data)
 {
 	struct gsm_lchan *lchan = lchan_fi_lchan(fi);
+	bool requires_rtp_stream = bsc_chan_ind_requires_rtp_stream(lchan->activate.info.ch_indctr);
+
 	switch (event) {
 
 	case LCHAN_EV_RLL_ESTABLISH_IND:
-		if (!lchan->activate.info.requires_voice_stream
-		    || lchan_rtp_established(lchan)) {
+		if (!requires_rtp_stream || lchan_rtp_established(lchan)) {
 			LOG_LCHAN(lchan, LOGL_DEBUG,
 				  "%s\n",
-				  (lchan->activate.info.requires_voice_stream ?
+				  (requires_rtp_stream ?
 				   "RTP already established earlier" : "no voice stream required"));
 			lchan_fsm_state_chg(LCHAN_ST_ESTABLISHED);
 		}
 		return;
 
 	case LCHAN_EV_RTP_READY:
-		if (lchan->sapis[0] != LCHAN_SAPI_UNUSED)
+		/* If RLL was established or if it does not need to be establised, because of VGCS/VBS channel. */
+		if (lchan->sapis[0] != LCHAN_SAPI_UNUSED ||
+		    lchan->activate.info.activ_for == ACTIVATE_FOR_VGCS_CHANNEL)
 			lchan_fsm_state_chg(LCHAN_ST_ESTABLISHED);
 		return;
 
@@ -1121,19 +1195,19 @@ static void lchan_fsm_wait_rsl_chan_mode_modify_ack(struct osmo_fsm_inst *fi, ui
 		/* The Channel Mode Modify was ACKed, now the requested values become the accepted and used values. */
 		lchan->current_ch_mode_rate = lchan->modify.ch_mode_rate;
 		lchan->current_mr_conf = lchan->modify.mr_conf_filtered;
+		lchan->current_ch_indctr = lchan->modify.ch_indctr;
 		lchan->tsc_set = lchan->modify.tsc_set;
 		lchan->tsc = lchan->modify.tsc;
-		lchan->vamos.enabled = lchan->modify.info.vamos;
+		lchan->vamos.enabled = (lchan->modify.info.type_for == LCHAN_TYPE_FOR_VAMOS);
 
-		if (lchan->modify.info.requires_voice_stream
-		    && !lchan->fi_rtp) {
+		if (bsc_chan_ind_requires_rtp_stream(lchan->modify.info.ch_indctr) && !lchan->fi_rtp) {
 			/* Continue with RTP stream establishing as done in lchan_activate(). Place the requested values in
 			 * lchan->activate.info and continue with voice stream setup. */
 			lchan->activate.info = (struct lchan_activate_info){
 				.activ_for = ACTIVATE_FOR_MODE_MODIFY_RTP,
 				.for_conn = lchan->conn,
 				.ch_mode_rate = lchan->modify.ch_mode_rate,
-				.requires_voice_stream = true,
+				.ch_indctr = lchan->modify.info.ch_indctr,
 				.msc_assigned_cic = lchan->modify.info.msc_assigned_cic,
 			};
 			if (lchan_activate_set_ch_mode_rate_and_mr_config(lchan))
@@ -1223,8 +1297,13 @@ static void handle_rll_rel_ind_or_conf(struct osmo_fsm_inst *fi, uint32_t event,
 
 	/* Releasing SAPI 0 means the conn becomes invalid; but not if the link_id contains a SACCH flag. */
 	if (lchan->conn && sapi == 0 && !(link_id & 0xc0)) {
-		LOG_LCHAN(lchan, LOGL_DEBUG, "lchan is releasing\n");
-		gscon_lchan_releasing(lchan->conn, lchan);
+		/* A VGCS/VBS channel must stay active, even if all SAPIs are released.
+		 * When a talker releases, the channel is available for the listeners and the next talker. The actual
+		 * channel release is performed by the VGCS/VBS call control. */
+		if (!lchan_is_asci(lchan)) {
+			LOG_LCHAN(lchan, LOGL_DEBUG, "lchan is releasing\n");
+			gscon_lchan_releasing(lchan->conn, lchan);
+		}
 
 		/* if SAPI=0 is gone, it makes no sense if other SAPIs are still around,
 		 * this is not a valid configuration and we should forget about them.
@@ -1242,6 +1321,7 @@ static void lchan_fsm_established(struct osmo_fsm_inst *fi, uint32_t event, void
 	struct gsm_lchan *lchan = lchan_fi_lchan(fi);
 	struct lchan_modify_info *modif_info;
 	struct osmo_mgcpc_ep_ci *use_mgwep_ci;
+	bool requires_rtp_stream = bsc_chan_ind_requires_rtp_stream(lchan->modify.info.ch_indctr);
 
 	switch (event) {
 	case LCHAN_EV_RLL_ESTABLISH_IND:
@@ -1251,7 +1331,8 @@ static void lchan_fsm_established(struct osmo_fsm_inst *fi, uint32_t event, void
 	case LCHAN_EV_RLL_REL_IND:
 	case LCHAN_EV_RLL_REL_CONF:
 		handle_rll_rel_ind_or_conf(fi, event, data);
-		if (!lchan_active_sapis(lchan, 0))
+		/* Only release channel, if there is no SAPI and this channel is not a VGCS channel. */
+		if (!lchan_active_sapis(lchan, 0) && !lchan_is_asci(lchan))
 			lchan_fsm_state_chg(LCHAN_ST_WAIT_RLL_RTP_RELEASED);
 		return;
 
@@ -1276,9 +1357,9 @@ static void lchan_fsm_established(struct osmo_fsm_inst *fi, uint32_t event, void
 		use_mgwep_ci = lchan_use_mgw_endpoint_ci_bts(lchan);
 
 		lchan->modify.ch_mode_rate = lchan->modify.info.ch_mode_rate;
-		lchan->modify.ch_mode_rate.chan_mode = (lchan->modify.info.vamos
-						? gsm48_chan_mode_to_vamos(lchan->modify.info.ch_mode_rate.chan_mode)
-						: gsm48_chan_mode_to_non_vamos(lchan->modify.info.ch_mode_rate.chan_mode));
+		lchan->modify.ch_mode_rate.chan_mode = (lchan->modify.info.type_for == LCHAN_TYPE_FOR_VAMOS)
+			? gsm48_chan_mode_to_vamos(lchan->modify.info.ch_mode_rate.chan_mode)
+			: gsm48_chan_mode_to_non_vamos(lchan->modify.info.ch_mode_rate.chan_mode);
 		if (lchan->modify.ch_mode_rate.chan_mode < 0) {
 			lchan_fail("Invalid chan_mode: %s", gsm48_chan_mode_name(lchan->modify.info.ch_mode_rate.chan_mode));
 			return;
@@ -1292,6 +1373,8 @@ static void lchan_fsm_established(struct osmo_fsm_inst *fi, uint32_t event, void
 			}
 		}
 
+		lchan->modify.ch_indctr = lchan->modify.info.ch_indctr;
+
 		/* If enabling VAMOS mode and no specific TSC Set was selected, make sure to select a sane TSC Set by
 		 * default: Set 1 for the primary and Set 2 for the shadow lchan. For non-VAMOS lchans, TSC Set 1. */
 		if (lchan->modify.info.tsc_set.present)
@@ -1304,10 +1387,10 @@ static void lchan_fsm_established(struct osmo_fsm_inst *fi, uint32_t event, void
 		lchan->modify.tsc = lchan->modify.info.tsc.present ? lchan->modify.info.tsc.val : gsm_ts_tsc(lchan->ts);
 
 		LOG_LCHAN(lchan, LOGL_INFO,
-			  "Modification requested: %s voice=%s MGW-ci=%s type=%s tch-mode=%s tsc=%d/%u\n",
+			  "Modification requested: %s rtp=%s MGW-ci=%s type=%s tch-mode=%s tsc=%d/%u\n",
 			  lchan_modify_for_name(lchan->modify.info.modify_for),
-			  lchan->modify.info.requires_voice_stream ? "yes" : "no",
-			  lchan->modify.info.requires_voice_stream ?
+			  requires_rtp_stream ? "yes" : "no",
+			  requires_rtp_stream ?
 			  (use_mgwep_ci ? osmo_mgcpc_ep_ci_name(use_mgwep_ci) : "new")
 			  : "none",
 			  gsm_chan_t_name(lchan->type),
@@ -1339,8 +1422,14 @@ static bool should_sacch_deact(struct gsm_lchan *lchan)
 
 static void lchan_do_release(struct gsm_lchan *lchan)
 {
-	if (lchan->release.do_rr_release && lchan->sapis[0] != LCHAN_SAPI_UNUSED)
-		gsm48_send_rr_release(lchan);
+	if (lchan->release.do_rr_release) {
+		/* To main DCCH in dedicated and group transmit mode */
+		if (lchan->sapis[0] != LCHAN_SAPI_UNUSED)
+			gsm48_send_rr_release(lchan, false);
+		/* As UI to all listeners in group receive mode */
+		if (lchan_is_asci(lchan))
+			gsm48_send_rr_release(lchan, true);
+	}
 
 	if (lchan->fi_rtp)
 		osmo_fsm_inst_dispatch(lchan->fi_rtp, LCHAN_RTP_EV_RELEASE, 0);
@@ -1809,9 +1898,9 @@ static int lchan_fsm_timer_cb(struct osmo_fsm_inst *fi)
 		lchan->release.rsl_error_cause = RSL_ERR_INTERWORKING;
 		lchan->release.rr_cause = bsc_gsm48_rr_cause_from_rsl_cause(lchan->release.rsl_error_cause);
 		if (fi->state  == LCHAN_ST_WAIT_RLL_RTP_ESTABLISH) {
-			lchan_fail("Timeout (rll_ready=%s,voice_require=%s,voice_ready=%s)",
+			lchan_fail("Timeout (rll_ready=%s,rtp_require=%s,voice_ready=%s)",
 				   (lchan->sapis[0] != LCHAN_SAPI_UNUSED) ? "yes" : "no",
-				   lchan->activate.info.requires_voice_stream ? "yes" : "no",
+				   bsc_chan_ind_requires_rtp_stream(lchan->activate.info.ch_indctr) ? "yes" : "no",
 				   lchan_rtp_established(lchan) ? "yes" : "no");
 		} else {
 			 lchan_fail("Timeout");
