@@ -1,6 +1,6 @@
 /* NM BTS FSM */
 
-/* (C) 2020 by sysmocom - s.m.f.c. GmbH <info@sysmocom.de>
+/* (C) 2020 by sysmocom - s.f.m.c. GmbH <info@sysmocom.de>
  * Author: Pau Espin Pedrol <pespin@sysmocom.de>
  *
  * All Rights Reserved
@@ -100,7 +100,7 @@ static void configure_loop(struct gsm_bts *bts, const struct gsm_nm_state *state
 	if (!bts->mo.get_attr_sent && !bts->mo.get_attr_rep_received) {
 		bts->mo.get_attr_sent = true;
 		/* N. B: we rely on attribute order when parsing response in abis_nm_rx_get_attr_resp() */
-		const uint8_t bts_attr[] = { NM_ATT_MANUF_ID, NM_ATT_SW_CONFIG, };
+		const uint8_t bts_attr[] = { NM_ATT_MANUF_ID, NM_ATT_SW_CONFIG, NM_ATT_IPACC_SUPP_FEATURES};
 		/* we should not request more attributes than we're ready to handle */
 		OSMO_ASSERT(sizeof(bts_attr) < MAX_BTS_ATTR);
 		abis_nm_get_attr(bts, NM_OC_BTS, 0, 0xff, 0xff,
@@ -122,7 +122,7 @@ static void configure_loop(struct gsm_bts *bts, const struct gsm_nm_state *state
 				      bts->bts_nr, 0xff, 0xff,
 				      NM_STATE_UNLOCKED);
 		/* Message containing BTS attributes, including the interference band bounds, was ACKed by the BTS.
-		 * Store the sent bounds as the ones being used for logging and comparing intereference levels. */
+		 * Store the sent bounds as the ones being used for logging and comparing interference levels. */
 		bts->interf_meas_params_used = bts->interf_meas_params_cfg;
 	}
 
@@ -286,6 +286,13 @@ static void st_op_enabled_on_enter(struct osmo_fsm_inst *fi, uint32_t prev_state
 	bts->mo.get_attr_rep_received = false;
 	bts->mo.set_attr_sent = false;
 	bts->mo.set_attr_ack_received = false;
+
+	/* Resume power saving on the BCCH carrier, if was enabled */
+	if (bts->c0_max_power_red_db > 0) {
+		LOG_BTS(bts, DRSL, LOGL_NOTICE, "Resuming BCCH carrier power reduction "
+			"operation mode (maximum %u dB)\n", bts->c0_max_power_red_db);
+		gsm_bts_send_c0_power_red(bts, bts->c0_max_power_red_db);
+	}
 }
 
 static void st_op_enabled(struct osmo_fsm_inst *fi, uint32_t event, void *data)
