@@ -36,6 +36,7 @@
 #include <osmocom/bsc/gsm_data.h>
 #include <osmocom/bsc/bts.h>
 #include <osmocom/bsc/paging.h>
+#include <osmocom/bsc/bssmap_reset.h>
 #include <osmocom/mgcp_client/mgcp_common.h>
 
 /* A pointer to a list with all involved MSCs
@@ -45,33 +46,6 @@ static struct llist_head *msc_list;
 #define DEFAULT_ASP_LOCAL_IP "localhost"
 #define DEFAULT_ASP_REMOTE_IP "localhost"
 
-/* Helper function to Check if the given connection id is already assigned */
-static struct gsm_subscriber_connection *get_bsc_conn_by_conn_id(int conn_id)
-{
-	conn_id &= 0xFFFFFF;
-	struct gsm_subscriber_connection *conn;
-
-	llist_for_each_entry(conn, &bsc_gsmnet->subscr_conns, entry) {
-		if (conn->sccp.conn_id == conn_id)
-			return conn;
-	}
-
-	return NULL;
-}
-
-struct gsm_subscriber_connection *bsc_conn_by_bsub(struct bsc_subscr *bsub)
-{
-	struct gsm_subscriber_connection *conn;
-	if (!bsub)
-		return NULL;
-
-	llist_for_each_entry(conn, &bsc_gsmnet->subscr_conns, entry) {
-		if (conn->bsub == bsub)
-			return conn;
-	}
-	return NULL;
-}
-
 /* Patch regular BSSMAP RESET to add extra T to announce Osmux support (osmocom extension) */
 static void _gsm0808_extend_announce_osmux(struct msgb *msg)
 {
@@ -94,7 +68,7 @@ void osmo_bsc_sigtran_tx_reset(const struct bsc_msc_data *msc)
 	if (msc_is_aoip(msc) && msc->use_osmux != OSMUX_USAGE_OFF)
 		_gsm0808_extend_announce_osmux(msg);
 
-	rate_ctr_inc(&msc->msc_ctrs->ctr[MSC_CTR_BSSMAP_TX_UDT_RESET]);
+	rate_ctr_inc(rate_ctr_group_get_ctr(msc->msc_ctrs, MSC_CTR_BSSMAP_TX_UDT_RESET));
 	osmo_sccp_tx_unitdata_msg(msc->a.sccp_user, &msc->a.bsc_addr,
 				  &msc->a.msc_addr, msg);
 }
@@ -114,12 +88,12 @@ void osmo_bsc_sigtran_tx_reset_ack(const struct bsc_msc_data *msc)
 	if (msc_is_aoip(msc) && msc->use_osmux != OSMUX_USAGE_OFF)
 		_gsm0808_extend_announce_osmux(msg);
 
-	rate_ctr_inc(&msc->msc_ctrs->ctr[MSC_CTR_BSSMAP_TX_UDT_RESET_ACK]);
+	rate_ctr_inc(rate_ctr_group_get_ctr(msc->msc_ctrs, MSC_CTR_BSSMAP_TX_UDT_RESET_ACK));
 	osmo_sccp_tx_unitdata_msg(msc->a.sccp_user, &msc->a.bsc_addr,
 				  &msc->a.msc_addr, msg);
 }
 
-/* Find an MSC by its sigtran point code */
+/* Find an MSC by its remote SCCP address */
 static struct bsc_msc_data *get_msc_by_addr(const struct osmo_sccp_addr *msc_addr)
 {
 	struct osmo_ss7_instance *ss7;
@@ -134,6 +108,21 @@ static struct bsc_msc_data *get_msc_by_addr(const struct osmo_sccp_addr *msc_add
 	return NULL;
 }
 
+/* Find an MSC by its remote sigtran point code on a given cs7 instance. */
+static struct bsc_msc_data *get_msc_by_pc(struct osmo_ss7_instance *cs7, uint32_t pc)
+{
+	struct bsc_msc_data *msc;
+	llist_for_each_entry(msc, msc_list, entry) {
+		if (msc->a.cs7_instance != cs7->cfg.id)
+			continue;
+		if ((msc->a.msc_addr.presence & OSMO_SCCP_ADDR_T_PC) == 0)
+			continue;
+		if (msc->a.msc_addr.pc == pc)
+			return msc;
+	}
+	return NULL;
+}
+
 /* Received data from MSC, use the connection id which MSC it is */
 static int handle_data_from_msc(struct gsm_subscriber_connection *conn, struct msgb *msg)
 {
@@ -164,10 +153,12 @@ static int handle_unitdata_from_msc(const struct osmo_sccp_addr *msc_addr, struc
 static int handle_n_connect_from_msc(struct osmo_sccp_user *scu, struct osmo_scu_prim *scu_prim)
 {
 	struct bsc_msc_data *msc = get_msc_by_addr(&scu_prim->u.connect.calling_addr);
+	struct osmo_sccp_instance *sccp = osmo_sccp_get_sccp(scu);
+	struct bsc_sccp_inst *bsc_sccp = osmo_sccp_get_priv(sccp);
 	struct gsm_subscriber_connection *conn;
 	int rc = 0;
 
-	conn = get_bsc_conn_by_conn_id(scu_prim->u.connect.conn_id);
+	conn = bsc_sccp_inst_get_gscon_by_conn_id(bsc_sccp, scu_prim->u.connect.conn_id);
 	if (conn) {
 		LOGP(DMSC, LOGL_NOTICE,
 		     "(calling_addr=%s conn_id=%u) N-CONNECT.ind with already used conn_id, ignoring\n",
@@ -195,7 +186,14 @@ static int handle_n_connect_from_msc(struct osmo_sccp_user *scu, struct osmo_scu
 	if (!conn)
 		return -ENOMEM;
 	conn->sccp.msc = msc;
-	conn->sccp.conn_id = scu_prim->u.connect.conn_id;
+	conn->sccp.conn.conn_id = scu_prim->u.connect.conn_id;
+	if (bsc_sccp_inst_register_gscon(bsc_sccp, &conn->sccp.conn) < 0) {
+		LOGP(DMSC, LOGL_NOTICE, "(calling_addr=%s conn_id=%u) N-CONNECT.ind failed registering conn\n",
+		     osmo_sccp_addr_dump(&scu_prim->u.connect.calling_addr), scu_prim->u.connect.conn_id);
+		osmo_fsm_inst_term(conn->fi, OSMO_FSM_TERM_REQUEST, NULL);
+		rc = -ENOENT;
+		goto refuse;
+	}
 
 	/* Take actions asked for by the enclosed PDU */
 	osmo_fsm_inst_dispatch(conn->fi, GSCON_EV_A_CONN_IND, scu_prim);
@@ -206,11 +204,101 @@ refuse:
 	return rc;
 }
 
+static void handle_pcstate_ind(struct osmo_ss7_instance *cs7, const struct osmo_scu_pcstate_param *pcst)
+{
+	struct bsc_msc_data *msc;
+	bool connected;
+	bool disconnected;
+
+	LOGP(DMSC, LOGL_DEBUG, "N-PCSTATE ind: affected_pc=%u sp_status=%s remote_sccp_status=%s\n",
+	     pcst->affected_pc, osmo_sccp_sp_status_name(pcst->sp_status),
+	     osmo_sccp_rem_sccp_status_name(pcst->remote_sccp_status));
+
+	/* If we don't care about that point-code, ignore PCSTATE. */
+	msc = get_msc_by_pc(cs7, pcst->affected_pc);
+	if (!msc)
+		return;
+
+	/* See if this marks the point code to have become available, or to have been lost.
+	 *
+	 * I want to detect two events:
+	 * - connection event (both indicators say PC is reachable).
+	 * - disconnection event (at least one indicator says the PC is not reachable).
+	 *
+	 * There are two separate incoming indicators with various possible values -- the incoming events can be:
+	 *
+	 * - neither connection nor disconnection indicated -- just indicating congestion
+	 *   connected == false, disconnected == false --> do nothing.
+	 * - both incoming values indicate that we are connected
+	 *   --> trigger connected
+	 * - both indicate we are disconnected
+	 *   --> trigger disconnected
+	 * - one value indicates 'connected', the other indicates 'disconnected'
+	 *   --> trigger disconnected
+	 *
+	 * Congestion could imply that we're connected, but it does not indicate that a PC's reachability changed, so no need to
+	 * trigger on that.
+	 */
+	connected = false;
+	disconnected = false;
+
+	switch (pcst->sp_status) {
+	case OSMO_SCCP_SP_S_ACCESSIBLE:
+		connected = true;
+		break;
+	case OSMO_SCCP_SP_S_INACCESSIBLE:
+		disconnected = true;
+		break;
+	default:
+	case OSMO_SCCP_SP_S_CONGESTED:
+		/* Neither connecting nor disconnecting */
+		break;
+	}
+
+	switch (pcst->remote_sccp_status) {
+	case OSMO_SCCP_REM_SCCP_S_AVAILABLE:
+		if (!disconnected)
+			connected = true;
+		break;
+	case OSMO_SCCP_REM_SCCP_S_UNAVAILABLE_UNKNOWN:
+	case OSMO_SCCP_REM_SCCP_S_UNEQUIPPED:
+	case OSMO_SCCP_REM_SCCP_S_INACCESSIBLE:
+		disconnected = true;
+		connected = false;
+		break;
+	default:
+	case OSMO_SCCP_REM_SCCP_S_CONGESTED:
+		/* Neither connecting nor disconnecting */
+		break;
+	}
+
+	if (disconnected && a_reset_conn_ready(msc)) {
+		LOGP(DMSC, LOGL_NOTICE,
+		     "(msc%d) now unreachable: N-PCSTATE ind: pc=%u sp_status=%s remote_sccp_status=%s\n",
+		     msc->nr, pcst->affected_pc,
+		     osmo_sccp_sp_status_name(pcst->sp_status),
+		     osmo_sccp_rem_sccp_status_name(pcst->remote_sccp_status));
+		/* A previously usable MSC has disconnected. Kick the BSSMAP back to DISC state. */
+		bssmap_reset_set_disconnected(msc->a.bssmap_reset);
+	} else if (connected && !a_reset_conn_ready(msc)) {
+		LOGP(DMSC, LOGL_NOTICE,
+		     "(msc%d) now available: N-PCSTATE ind: pc=%u sp_status=%s remote_sccp_status=%s\n",
+		     msc->nr, pcst->affected_pc,
+		     osmo_sccp_sp_status_name(pcst->sp_status),
+		     osmo_sccp_rem_sccp_status_name(pcst->remote_sccp_status));
+		/* A previously unusable MSC has become reachable. Trigger immediate BSSMAP RESET -- we would resend a
+		 * RESET either way, but we might as well do it now to speed up connecting. */
+		bssmap_reset_resend_reset(msc->a.bssmap_reset);
+	}
+}
+
 /* Callback function, called by the SCCP stack when data arrives */
 static int sccp_sap_up(struct osmo_prim_hdr *oph, void *_scu)
 {
 	struct osmo_scu_prim *scu_prim = (struct osmo_scu_prim *)oph;
 	struct osmo_sccp_user *scu = _scu;
+	struct osmo_sccp_instance *sccp = osmo_sccp_get_sccp(scu);
+	struct bsc_sccp_inst *bsc_sccp = osmo_sccp_get_priv(sccp);
 	struct gsm_subscriber_connection *conn;
 	int rc = 0;
 
@@ -231,7 +319,7 @@ static int sccp_sap_up(struct osmo_prim_hdr *oph, void *_scu)
 		/* Handle outbound connection confirmation */
 		DEBUGP(DMSC, "N-CONNECT.cnf(%u, %s)\n", scu_prim->u.connect.conn_id,
 		       osmo_hexdump(msgb_l2(oph->msg), msgb_l2len(oph->msg)));
-		conn = get_bsc_conn_by_conn_id(scu_prim->u.connect.conn_id);
+		conn = bsc_sccp_inst_get_gscon_by_conn_id(bsc_sccp, scu_prim->u.connect.conn_id);
 		if (conn) {
 			osmo_fsm_inst_dispatch(conn->fi, GSCON_EV_A_CONN_CFM, scu_prim);
 			conn->sccp.state = SUBSCR_SCCP_ST_CONNECTED;
@@ -250,7 +338,7 @@ static int sccp_sap_up(struct osmo_prim_hdr *oph, void *_scu)
 		       osmo_hexdump(msgb_l2(oph->msg), msgb_l2len(oph->msg)));
 
 		/* Incoming data is a sign of a vital connection */
-		conn = get_bsc_conn_by_conn_id(scu_prim->u.data.conn_id);
+		conn = bsc_sccp_inst_get_gscon_by_conn_id(bsc_sccp, scu_prim->u.data.conn_id);
 		if (conn) {
 			a_reset_conn_success(conn->sccp.msc);
 			handle_data_from_msc(conn, oph->msg);
@@ -262,7 +350,7 @@ static int sccp_sap_up(struct osmo_prim_hdr *oph, void *_scu)
 		       osmo_hexdump(msgb_l2(oph->msg), msgb_l2len(oph->msg)),
 		       scu_prim->u.disconnect.cause);
 		/* indication of disconnect */
-		conn = get_bsc_conn_by_conn_id(scu_prim->u.disconnect.conn_id);
+		conn = bsc_sccp_inst_get_gscon_by_conn_id(bsc_sccp, scu_prim->u.disconnect.conn_id);
 		if (conn) {
 			conn->sccp.state = SUBSCR_SCCP_ST_NONE;
 			if (msgb_l2len(oph->msg) > 0)
@@ -271,6 +359,10 @@ static int sccp_sap_up(struct osmo_prim_hdr *oph, void *_scu)
 		}
 		break;
 
+	case OSMO_PRIM(OSMO_SCU_PRIM_N_PCSTATE, PRIM_OP_INDICATION):
+		handle_pcstate_ind(osmo_sccp_get_ss7(sccp), &scu_prim->u.pcstate);
+		break;
+
 	default:
 		LOGP(DMSC, LOGL_ERROR, "Unhandled SIGTRAN operation %s on primitive %u\n",
 		     get_value_string(osmo_prim_op_names, oph->operation), oph->primitive);
@@ -312,17 +404,19 @@ enum bsc_con osmo_bsc_sigtran_new_conn(struct gsm_subscriber_connection *conn, s
 }
 
 /* Open a new connection oriented sigtran connection */
-int osmo_bsc_sigtran_open_conn(struct gsm_subscriber_connection *conn, struct msgb *msg)
+/* Allow test to overwrite it */
+__attribute__((weak)) int osmo_bsc_sigtran_open_conn(struct gsm_subscriber_connection *conn, struct msgb *msg)
 {
 	struct osmo_ss7_instance *ss7;
 	struct bsc_msc_data *msc;
-	int conn_id;
+	struct bsc_sccp_inst *bsc_sccp;
+	uint32_t conn_id;
 	int rc;
 
 	OSMO_ASSERT(conn);
 	OSMO_ASSERT(msg);
 	OSMO_ASSERT(conn->sccp.msc);
-	OSMO_ASSERT(conn->sccp.conn_id == -1);
+	OSMO_ASSERT(conn->sccp.conn.conn_id == SCCP_CONN_ID_UNSET);
 
 	msc = conn->sccp.msc;
 
@@ -331,15 +425,20 @@ int osmo_bsc_sigtran_open_conn(struct gsm_subscriber_connection *conn, struct ms
 		return -EINVAL;
 	}
 
-	conn->sccp.conn_id = conn_id = bsc_sccp_inst_next_conn_id(conn->sccp.msc->a.sccp);
-	if (conn->sccp.conn_id < 0) {
+	bsc_sccp = osmo_sccp_get_priv(msc->a.sccp);
+	conn->sccp.conn.conn_id = conn_id = bsc_sccp_inst_next_conn_id(bsc_sccp);
+	if (conn->sccp.conn.conn_id == SCCP_CONN_ID_UNSET) {
 		LOGP(DMSC, LOGL_ERROR, "Unable to allocate SCCP Connection ID\n");
 		return -1;
 	}
-	LOGP(DMSC, LOGL_DEBUG, "Allocated new connection id: %d\n", conn->sccp.conn_id);
+	if (bsc_sccp_inst_register_gscon(bsc_sccp, &conn->sccp.conn) < 0) {
+		LOGP(DMSC, LOGL_ERROR, "Unable to register SCCP connection (id=%u)\n", conn->sccp.conn.conn_id);
+		return -1;
+	}
+	LOGP(DMSC, LOGL_DEBUG, "Allocated new connection id: %u\n", conn->sccp.conn.conn_id);
 	ss7 = osmo_ss7_instance_find(msc->a.cs7_instance);
 	OSMO_ASSERT(ss7);
-	LOGP(DMSC, LOGL_INFO, "Opening new SCCP connection (id=%i) to MSC %d: %s\n", conn_id,
+	LOGP(DMSC, LOGL_INFO, "Opening new SCCP connection (id=%u) to MSC %d: %s\n", conn_id,
 	     msc->nr, osmo_sccp_addr_name(ss7, &msc->a.msc_addr));
 
 	rc = osmo_sccp_tx_conn_req_msg(msc->a.sccp_user, conn_id, &msc->a.bsc_addr,
@@ -374,32 +473,32 @@ int osmo_bsc_sigtran_send(struct gsm_subscriber_connection *conn, struct msgb *m
 	if (msg->len >= 3) {
 		switch (msg->data[0]) {
 		case BSSAP_MSG_BSS_MANAGEMENT:
-			rate_ctr_inc(&msc->msc_ctrs->ctr[MSC_CTR_BSSMAP_TX_BSS_MANAGEMENT]);
+			rate_ctr_inc(rate_ctr_group_get_ctr(msc->msc_ctrs, MSC_CTR_BSSMAP_TX_BSS_MANAGEMENT));
 			LOGP(DMSC, LOGL_INFO, "Tx MSC: BSSMAP: %s\n",
 			     gsm0808_bssmap_name(msg->data[2]));
 			break;
 		case BSSAP_MSG_DTAP:
-			rate_ctr_inc(&msc->msc_ctrs->ctr[MSC_CTR_BSSMAP_TX_DTAP]);
+			rate_ctr_inc(rate_ctr_group_get_ctr(msc->msc_ctrs, MSC_CTR_BSSMAP_TX_DTAP));
 			LOGP(DMSC, LOGL_INFO, "Tx MSC: DTAP\n");
 			break;
 		default:
-			rate_ctr_inc(&msc->msc_ctrs->ctr[MSC_CTR_BSSMAP_TX_UNKNOWN]);
+			rate_ctr_inc(rate_ctr_group_get_ctr(msc->msc_ctrs, MSC_CTR_BSSMAP_TX_UNKNOWN));
 			LOGP(DMSC, LOGL_ERROR, "Tx MSC: unknown message type: 0x%x\n",
 			     msg->data[0]);
 		}
 	} else {
-		rate_ctr_inc(&msc->msc_ctrs->ctr[MSC_CTR_BSSMAP_TX_SHORT]);
+		rate_ctr_inc(rate_ctr_group_get_ctr(msc->msc_ctrs, MSC_CTR_BSSMAP_TX_SHORT));
 		LOGP(DMSC, LOGL_ERROR, "Tx MSC: message too short: %u\n", msg->len);
 	}
 
 	if (a_reset_conn_ready(msc) == false) {
-		rate_ctr_inc(&msc->msc_ctrs->ctr[MSC_CTR_BSSMAP_TX_ERR_CONN_NOT_READY]);
+		rate_ctr_inc(rate_ctr_group_get_ctr(msc->msc_ctrs, MSC_CTR_BSSMAP_TX_ERR_CONN_NOT_READY));
 		LOGP(DMSC, LOGL_ERROR, "MSC is not connected. Dropping.\n");
 		msgb_free(msg);
 		return -EINVAL;
 	}
 
-	conn_id = conn->sccp.conn_id;
+	conn_id = conn->sccp.conn.conn_id;
 
 	ss7 = osmo_ss7_instance_find(msc->a.cs7_instance);
 	OSMO_ASSERT(ss7);
@@ -408,9 +507,9 @@ int osmo_bsc_sigtran_send(struct gsm_subscriber_connection *conn, struct msgb *m
 
 	rc = osmo_sccp_tx_data_msg(msc->a.sccp_user, conn_id, msg);
 	if (rc >= 0)
-		rate_ctr_inc(&msc->msc_ctrs->ctr[MSC_CTR_BSSMAP_TX_SUCCESS]);
+		rate_ctr_inc(rate_ctr_group_get_ctr(msc->msc_ctrs, MSC_CTR_BSSMAP_TX_SUCCESS));
 	else
-		rate_ctr_inc(&msc->msc_ctrs->ctr[MSC_CTR_BSSMAP_TX_ERR_SEND]);
+		rate_ctr_inc(rate_ctr_group_get_ctr(msc->msc_ctrs, MSC_CTR_BSSMAP_TX_ERR_SEND));
 
 	return rc;
 }
@@ -513,6 +612,7 @@ int osmo_bsc_sigtran_init(struct llist_head *mscs)
 		int prev_msc_nr;
 
 		struct osmo_sccp_instance *sccp;
+		struct bsc_sccp_inst *bsc_sccp;
 
 		llist_for_each_entry(msc, msc_list, entry) {
 			/* An MSC with invalid cs7 instance id defaults to cs7 instance 0 */
@@ -556,6 +656,10 @@ int osmo_bsc_sigtran_init(struct llist_head *mscs)
 		if (!sccp)
 			return -EINVAL;
 
+		bsc_sccp = bsc_sccp_inst_alloc(tall_bsc_ctx);
+		bsc_sccp->sccp = sccp;
+		osmo_sccp_set_priv(sccp, bsc_sccp);
+
 		/* Now that the SCCP client is set up, configure all MSCs on this cs7 instance to use it */
 		llist_for_each_entry(msc, msc_list, entry) {
 			char msc_name[32];
@@ -625,7 +729,7 @@ static int asp_rx_unknown(struct osmo_ss7_asp *asp, int ppid_mux, struct msgb *m
 	struct ipaccess_head *iph;
 	struct ipaccess_head_ext *iph_ext;
 
-	if (asp->cfg.proto != OSMO_SS7_ASP_PROT_IPA) {
+	if (osmo_ss7_asp_get_proto(asp) != OSMO_SS7_ASP_PROT_IPA) {
 		msgb_free(msg);
 		return 0;
 	}
