@@ -50,8 +50,6 @@ static int pcu_sock_send(struct gsm_network *net, struct msgb *msg);
 
 static const char *sapi_string[] = {
 	[PCU_IF_SAPI_RACH] =	"RACH",
-	[PCU_IF_SAPI_AGCH] =	"AGCH",
-	[PCU_IF_SAPI_PCH] =	"PCH",
 	[PCU_IF_SAPI_BCCH] =	"BCCH",
 	[PCU_IF_SAPI_PDTCH] =	"PDTCH",
 	[PCU_IF_SAPI_PRACH] =	"PRACH",
@@ -200,7 +198,7 @@ static int pcu_tx_info_ind(struct gsm_bts *bts)
 	info_ind = &pcu_prim->u.info_ind;
 	info_ind->version = PCU_IF_VERSION;
 	info_ind->flags |= PCU_IF_FLAG_ACTIVE;
-	info_ind->flags |= PCU_IF_FLAG_SYSMO;
+	info_ind->flags |= PCU_IF_FLAG_DIRECT_PHY;
 
 	/* RAI */
 	info_ind->mcc = bts->network->plmn.mcc;
@@ -302,6 +300,14 @@ static int pcu_tx_info_ind(struct gsm_bts *bts)
 		info_ind_fill_trx(&info_ind->trx[trx->nr], trx);
 	}
 
+	switch (bts->type) {
+	case GSM_BTS_TYPE_RBS2000:
+		info_ind->bts_model = PCU_IF_BTS_MODEL_RBS;
+		break;
+	default:
+		info_ind->bts_model = PCU_IF_BTS_MODEL_UNSPEC;
+	}
+
 	return pcu_sock_send(bts->network, msg);
 }
 
