@@ -220,7 +220,16 @@ int bts_earfcn_add(struct gsm_bts *bts, uint16_t earfcn, uint8_t thresh_hi, uint
 		   uint8_t qrx, uint8_t meas_bw)
 {
 	struct osmo_earfcn_si2q *e = &bts->si_common.si2quater_neigh_list;
-	int r = osmo_earfcn_add(e, earfcn, (meas_bw < EARFCN_MEAS_BW_INVALID) ? meas_bw : OSMO_EARFCN_MEAS_INVALID);
+	int r;
+
+	/* EARFCN may already exist, so we delete it to avoid duplicates */
+	if (osmo_earfcn_del(e, earfcn) == 0)
+		LOGP(DRR, LOGL_NOTICE, "EARFCN %u is already in the list, modifying\n", earfcn);
+
+	if (meas_bw < EARFCN_MEAS_BW_INVALID)
+		r = osmo_earfcn_add(e, earfcn, meas_bw);
+	else
+		r = osmo_earfcn_add(e, earfcn, OSMO_EARFCN_MEAS_INVALID);
 
 	if (r < 0)
 		return r;
@@ -298,16 +307,21 @@ int bts_uarfcn_add(struct gsm_bts *bts, uint16_t arfcn, uint16_t scramble, bool
 	size_t len = bts->si_common.uarfcn_length, i;
 	uint8_t si2q;
 	int pos = uarfcn_sc_pos(bts, arfcn, scramble);
-	uint16_t scr = diversity ? encode_fdd(scramble, true) : encode_fdd(scramble, false),
+	uint16_t scr = encode_fdd(scramble, diversity),
 		*ual = bts->si_common.data.uarfcn_list,
 		*scl = bts->si_common.data.scramble_list;
 
+	if (pos >= 0) {
+		LOGP(DRR, LOGL_NOTICE,
+		     "EARFCN (%u, %u) is already in the list, modifying\n",
+		     arfcn, scramble);
+		scl[pos] = scr;
+		return 0;
+	}
+
 	if (len == MAX_EARFCN_LIST)
 		return -ENOMEM;
 
-	if (pos >= 0)
-		return -EADDRINUSE;
-
 	/* find the suitable position for arfcn if any */
 	pos = uarfcn_sc_pos(bts, arfcn, SC_BOUND);
 	i = (pos < 0) ? len : pos;
@@ -757,7 +771,19 @@ static int generate_si1(enum osmo_sysinfo_type t, struct gsm_bts *bts)
 	 * SI1 Rest Octets (10.5.2.32), contains NCH position and band
 	 * indicator but that is not in the 04.08.
 	 */
-	rc = osmo_gsm48_rest_octets_si1_encode(si1->rest_octets, NULL, is_dcs_net(bts));
+	if (bts->nch.num_blocks) {
+		rc = osmo_gsm48_si1ro_nch_pos_encode(bts->nch.num_blocks, bts->nch.first_block);
+		if (rc < 0) {
+			LOGP(DRR, LOGL_ERROR, "Unable to encode NCH position (num_blocks=%u, first_block=%u)\n",
+			     bts->nch.num_blocks, bts->nch.first_block);
+			rc = osmo_gsm48_rest_octets_si1_encode(si1->rest_octets, NULL, is_dcs_net(bts));
+		} else {
+			uint8_t nch_pos = rc;
+			rc = osmo_gsm48_rest_octets_si1_encode(si1->rest_octets, &nch_pos, is_dcs_net(bts));
+		}
+	} else {
+		rc = osmo_gsm48_rest_octets_si1_encode(si1->rest_octets, NULL, is_dcs_net(bts));
+	}
 
 	return sizeof(*si1) + rc;
 }
@@ -1069,8 +1095,8 @@ static int generate_si5(enum osmo_sysinfo_type t, struct gsm_bts *bts)
 
 	memset(output, GSM_MACBLOCK_PADDING, GSM_MACBLOCK_LEN);
 
-	/* ip.access nanoBTS needs l2_plen!! */
-	if (is_ipaccess_bts(bts)) {
+	/* Abis/IP needs l2_plen!! */
+	if (is_ipa_abisip_bts(bts)) {
 		*output++ = GSM48_LEN2PLEN(l2_plen);
 		l2_plen++;
 	}
@@ -1100,8 +1126,8 @@ static int generate_si5bis(enum osmo_sysinfo_type t, struct gsm_bts *bts)
 
 	memset(output, GSM_MACBLOCK_PADDING, GSM_MACBLOCK_LEN);
 
-	/* ip.access nanoBTS needs l2_plen!! */
-	if (is_ipaccess_bts(bts)) {
+	/* Abis/IP needs l2_plen!! */
+	if (is_ipa_abisip_bts(bts)) {
 		*output++ = GSM48_LEN2PLEN(l2_plen);
 		l2_plen++;
 	}
@@ -1139,8 +1165,8 @@ static int generate_si5ter(enum osmo_sysinfo_type t, struct gsm_bts *bts)
 
 	memset(output, GSM_MACBLOCK_PADDING, GSM_MACBLOCK_LEN);
 
-	/* ip.access nanoBTS needs l2_plen!! */
-	if (is_ipaccess_bts(bts)) {
+	/* Abis/IP needs l2_plen!! */
+	if (is_ipa_abisip_bts(bts)) {
 		*output++ = GSM48_LEN2PLEN(l2_plen);
 		l2_plen++;
 	}
@@ -1174,8 +1200,8 @@ static int generate_si6(enum osmo_sysinfo_type t, struct gsm_bts *bts)
 	memset(output, GSM_MACBLOCK_PADDING, GSM_MACBLOCK_LEN);
 	memset(&si6_ro_info, 0, sizeof(si6_ro_info));
 
-	/* ip.access nanoBTS needs l2_plen!! */
-	if (is_ipaccess_bts(bts)) {
+	/* Abis/IP needs l2_plen!! */
+	if (is_ipa_abisip_bts(bts)) {
 		*output++ = GSM48_LEN2PLEN(l2_plen);
 		l2_plen++;
 	}
@@ -1218,7 +1244,9 @@ static int generate_si13(enum osmo_sysinfo_type t, struct gsm_bts *bts)
 			.nmo		= GPRS_NMO_II,
 			.t3168		= 2000,
 			.t3192		= 1500,
-			.drx_timer_max	= 3,
+			/* 3GPP TS 45.002 6.5.6:
+			 * "On BCCH, the operator should limit DRX_TIMER_MAX [...] to 4 seconds" */
+			.drx_timer_max	= 4,
 			.bs_cv_max	= 15,
 			.ctrl_ack_type_use_block = bts->gprs.ctrl_ack_type_use_block,
 			.ext_info_present = true,
